#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .option_9 import option
from .cbk import cbk
from .kinetics_diffusion_limited import kinetics_diffusion_limited
from .intrinsic_model import intrinsic_model
from .multiple_surface_reactions import multiple_surface_reactions
class combustion_model(Group):
    """
    'combustion_model' child.
    """

    fluent_name = "combustion-model"

    child_names = \
        ['option', 'cbk', 'kinetics_diffusion_limited', 'intrinsic_model',
         'multiple_surface_reactions']

    option: option = option
    """
    option child of combustion_model.
    """
    cbk: cbk = cbk
    """
    cbk child of combustion_model.
    """
    kinetics_diffusion_limited: kinetics_diffusion_limited = kinetics_diffusion_limited
    """
    kinetics_diffusion_limited child of combustion_model.
    """
    intrinsic_model: intrinsic_model = intrinsic_model
    """
    intrinsic_model child of combustion_model.
    """
    multiple_surface_reactions: multiple_surface_reactions = multiple_surface_reactions
    """
    multiple_surface_reactions child of combustion_model.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .vof_smooth_options import vof_smooth_options
class patch(Group):
    """
    'patch' child.
    """

    fluent_name = "patch"

    child_names = \
        ['vof_smooth_options']

    vof_smooth_options: vof_smooth_options = vof_smooth_options
    """
    vof_smooth_options child of patch.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .host_file import host_file
class load_hosts(Command):
    """
    Read a hosts file.
    
    Parameters
    ----------
        host_file : str
            'host_file' child.
    
    """

    fluent_name = "load-hosts"

    argument_names = \
        ['host_file']

    host_file: host_file = host_file
    """
    host_file argument of load_hosts.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .option_2 import option
from .min_max import min_max
from .in_range import in_range
from .except_in_range import except_in_range
from .top_value_cells import top_value_cells
from .low_value_cells import low_value_cells
from .less_than import less_than
from .more_than import more_than
from .between_std_dev import between_std_dev
from .outside_std_dev import outside_std_dev
from .less_than_std_dev import less_than_std_dev
from .more_than_std_dev import more_than_std_dev
class option(Group):
    """
    'option' child.
    """

    fluent_name = "option"

    child_names = \
        ['option', 'min_max', 'in_range', 'except_in_range',
         'top_value_cells', 'low_value_cells', 'less_than', 'more_than',
         'between_std_dev', 'outside_std_dev', 'less_than_std_dev',
         'more_than_std_dev']

    option: option = option
    """
    option child of option.
    """
    min_max: min_max = min_max
    """
    min_max child of option.
    """
    in_range: in_range = in_range
    """
    in_range child of option.
    """
    except_in_range: except_in_range = except_in_range
    """
    except_in_range child of option.
    """
    top_value_cells: top_value_cells = top_value_cells
    """
    top_value_cells child of option.
    """
    low_value_cells: low_value_cells = low_value_cells
    """
    low_value_cells child of option.
    """
    less_than: less_than = less_than
    """
    less_than child of option.
    """
    more_than: more_than = more_than
    """
    more_than child of option.
    """
    between_std_dev: between_std_dev = between_std_dev
    """
    between_std_dev child of option.
    """
    outside_std_dev: outside_std_dev = outside_std_dev
    """
    outside_std_dev child of option.
    """
    less_than_std_dev: less_than_std_dev = less_than_std_dev
    """
    less_than_std_dev child of option.
    """
    more_than_std_dev: more_than_std_dev = more_than_std_dev
    """
    more_than_std_dev child of option.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .address import address
from .port import port
class start(Command):
    """
    'start' command.
    
    Parameters
    ----------
        address : str
            'address' child.
        port : int
            'port' child.
    
    """

    fluent_name = "start"

    argument_names = \
        ['address', 'port']

    address: address = address
    """
    address argument of start.
    """
    port: port = port
    """
    port argument of start.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class smoothing_relaxation_factor(Real):
    """
    Set Smoothing relaxation factor (min : 0, max : 1).
    """

    fluent_name = "smoothing-relaxation-factor"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .only_list_case_boundaries import only_list_case_boundaries
from .use_inherent_material_color import use_inherent_material_color
from .reset import reset
class by_type(Group):
    """
    'by_type' child.
    """

    fluent_name = "by-type"

    child_names = \
        ['only_list_case_boundaries', 'use_inherent_material_color']

    only_list_case_boundaries: only_list_case_boundaries = only_list_case_boundaries
    """
    only_list_case_boundaries child of by_type.
    """
    use_inherent_material_color: use_inherent_material_color = use_inherent_material_color
    """
    use_inherent_material_color child of by_type.
    """
    command_names = \
        ['reset']

    reset: reset = reset
    """
    reset command of by_type.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class re_randomize_every_timestep(Boolean):
    """
    Enable/disable changing the pseudo-random number sequences from timestep to timestep.
    """

    fluent_name = "re-randomize-every-timestep?"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .retain_instantaneous_values import retain_instantaneous_values
from .input_params import input_params
from .function_name import function_name
from .average_over import average_over
from .old_props import old_props
class user_defined_child(Group):
    """
    'child_object_type' of user_defined.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['retain_instantaneous_values', 'input_params', 'function_name',
         'average_over', 'old_props']

    retain_instantaneous_values: retain_instantaneous_values = retain_instantaneous_values
    """
    retain_instantaneous_values child of user_defined_child.
    """
    input_params: input_params = input_params
    """
    input_params child of user_defined_child.
    """
    function_name: function_name = function_name
    """
    function_name child of user_defined_child.
    """
    average_over: average_over = average_over
    """
    average_over child of user_defined_child.
    """
    old_props: old_props = old_props
    """
    old_props child of user_defined_child.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .summary_state import summary_state
from .reset_dpm_summaries import reset_dpm_summaries
class zone_summaries_per_injection(Command):
    """
    Enable per-injection zone DPM summaries.
    
    Parameters
    ----------
        summary_state : bool
            'summary_state' child.
        reset_dpm_summaries : bool
            'reset_dpm_summaries' child.
    
    """

    fluent_name = "zone-summaries-per-injection?"

    argument_names = \
        ['summary_state', 'reset_dpm_summaries']

    summary_state: summary_state = summary_state
    """
    summary_state argument of zone_summaries_per_injection.
    """
    reset_dpm_summaries: reset_dpm_summaries = reset_dpm_summaries
    """
    reset_dpm_summaries argument of zone_summaries_per_injection.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .viscosity_averaging import viscosity_averaging
from .turb_visc_based_damping import turb_visc_based_damping
from .density_func_expo import density_func_expo
from .density_ratio_cutoff import density_ratio_cutoff
from .interfacial_artificial_viscosity import interfacial_artificial_viscosity
class viscous_flow(Group):
    """
    'viscous_flow' child.
    """

    fluent_name = "viscous-flow"

    child_names = \
        ['viscosity_averaging', 'turb_visc_based_damping',
         'density_func_expo', 'density_ratio_cutoff',
         'interfacial_artificial_viscosity']

    viscosity_averaging: viscosity_averaging = viscosity_averaging
    """
    viscosity_averaging child of viscous_flow.
    """
    turb_visc_based_damping: turb_visc_based_damping = turb_visc_based_damping
    """
    turb_visc_based_damping child of viscous_flow.
    """
    density_func_expo: density_func_expo = density_func_expo
    """
    density_func_expo child of viscous_flow.
    """
    density_ratio_cutoff: density_ratio_cutoff = density_ratio_cutoff
    """
    density_ratio_cutoff child of viscous_flow.
    """
    interfacial_artificial_viscosity: interfacial_artificial_viscosity = interfacial_artificial_viscosity
    """
    interfacial_artificial_viscosity child of viscous_flow.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class dpm_upper_deposition_limit_offset(Real):
    """
    'dpm_upper_deposition_limit_offset' child.
    """

    fluent_name = "dpm-upper-deposition-limit-offset"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .report_files_child import report_files_child

class report_files(NamedObject[report_files_child], _CreatableNamedObjectMixin[report_files_child]):
    """
    'report_files' child.
    """

    fluent_name = "report-files"

    child_object_type: report_files_child = report_files_child
    """
    child_object_type of report_files.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .options_11 import options
from .domain_val import domain_val
from .all_wall_zones import all_wall_zones
from .wall_thread_list import wall_thread_list
from .direction_vector_1 import direction_vector
from .momentum_center import momentum_center
from .momentum_axis import momentum_axis
from .pressure_coordinate import pressure_coordinate
from .coord_val import coord_val
from .write_to_file import write_to_file
from .file_name_1 import file_name
from .append_data import append_data
from .overwrite import overwrite
class forces(Command):
    """
    'forces' command.
    
    Parameters
    ----------
        options : str
            'options' child.
        domain_val : str
            'domain_val' child.
        all_wall_zones : bool
            Select all wall zones available.
        wall_thread_list : typing.List[str]
            'wall_thread_list' child.
        direction_vector : typing.Tuple[real, real, real]
            'direction_vector' child.
        momentum_center : typing.Tuple[real, real, real]
            'momentum_center' child.
        momentum_axis : typing.Tuple[real, real, real]
            'momentum_axis' child.
        pressure_coordinate : str
            'pressure_coordinate' child.
        coord_val : real
            'coord_val' child.
        write_to_file : bool
            'write_to_file' child.
        file_name : str
            'file_name' child.
        append_data : bool
            'append_data' child.
        overwrite : bool
            'overwrite' child.
    
    """

    fluent_name = "forces"

    argument_names = \
        ['options', 'domain_val', 'all_wall_zones', 'wall_thread_list',
         'direction_vector', 'momentum_center', 'momentum_axis',
         'pressure_coordinate', 'coord_val', 'write_to_file', 'file_name',
         'append_data', 'overwrite']

    options: options = options
    """
    options argument of forces.
    """
    domain_val: domain_val = domain_val
    """
    domain_val argument of forces.
    """
    all_wall_zones: all_wall_zones = all_wall_zones
    """
    all_wall_zones argument of forces.
    """
    wall_thread_list: wall_thread_list = wall_thread_list
    """
    wall_thread_list argument of forces.
    """
    direction_vector: direction_vector = direction_vector
    """
    direction_vector argument of forces.
    """
    momentum_center: momentum_center = momentum_center
    """
    momentum_center argument of forces.
    """
    momentum_axis: momentum_axis = momentum_axis
    """
    momentum_axis argument of forces.
    """
    pressure_coordinate: pressure_coordinate = pressure_coordinate
    """
    pressure_coordinate argument of forces.
    """
    coord_val: coord_val = coord_val
    """
    coord_val argument of forces.
    """
    write_to_file: write_to_file = write_to_file
    """
    write_to_file argument of forces.
    """
    file_name: file_name = file_name
    """
    file_name argument of forces.
    """
    append_data: append_data = append_data
    """
    append_data argument of forces.
    """
    overwrite: overwrite = overwrite
    """
    overwrite argument of forces.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .enable_expert_view import enable_expert_view
from .general_settings import general_settings
from .injections import injections
from .numerics import numerics
from .parallel import parallel
from .physical_models_1 import physical_models
from .tracking_1 import tracking
from .user_defined_functions import user_defined_functions
class discrete_phase(Group):
    """
    Toplevel menu of the Discrete Phase multiphase model. A discrete phase model (DPM) is used when the aim is to investigate the behavior of the particles from a Lagrangian view and a discrete perspective.
    """

    fluent_name = "discrete-phase"

    child_names = \
        ['enable_expert_view', 'general_settings', 'injections', 'numerics',
         'parallel', 'physical_models', 'tracking', 'user_defined_functions']

    enable_expert_view: enable_expert_view = enable_expert_view
    """
    enable_expert_view child of discrete_phase.
    """
    general_settings: general_settings = general_settings
    """
    general_settings child of discrete_phase.
    """
    injections: injections = injections
    """
    injections child of discrete_phase.
    """
    numerics: numerics = numerics
    """
    numerics child of discrete_phase.
    """
    parallel: parallel = parallel
    """
    parallel child of discrete_phase.
    """
    physical_models: physical_models = physical_models
    """
    physical_models child of discrete_phase.
    """
    tracking: tracking = tracking
    """
    tracking child of discrete_phase.
    """
    user_defined_functions: user_defined_functions = user_defined_functions
    """
    user_defined_functions child of discrete_phase.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .hardcopy_format import hardcopy_format
from .hardcopy_options import hardcopy_options
from .window_dump_cmd import window_dump_cmd
from .post_format import post_format
from .current_driver import current_driver
class driver_options(Group):
    """
    'driver_options' child.
    """

    fluent_name = "driver-options"

    child_names = \
        ['hardcopy_format', 'hardcopy_options', 'window_dump_cmd',
         'post_format']

    hardcopy_format: hardcopy_format = hardcopy_format
    """
    hardcopy_format child of driver_options.
    """
    hardcopy_options: hardcopy_options = hardcopy_options
    """
    hardcopy_options child of driver_options.
    """
    window_dump_cmd: window_dump_cmd = window_dump_cmd
    """
    window_dump_cmd child of driver_options.
    """
    post_format: post_format = post_format
    """
    post_format child of driver_options.
    """
    command_names = \
        ['current_driver']

    current_driver: current_driver = current_driver
    """
    current_driver command of driver_options.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .always_use_face_centroid_with_periodics import always_use_face_centroid_with_periodics
from .interpolation import interpolation
from .boundary_layer_tracking import boundary_layer_tracking
from .check_subtet_validity import check_subtet_validity
from .use_automatic_intersection_tolerance import use_automatic_intersection_tolerance
from .use_barycentric_intersection import use_barycentric_intersection
from .particle_relocation import particle_relocation
from .remove_stuck_particles import remove_stuck_particles
from .use_barycentric_sampling import use_barycentric_sampling
from .use_quad_face_centroid import use_quad_face_centroid
class high_res_tracking_options(Group):
    """
    Main menu containing settings that control the High-Res Tracking scheme.
    When the High-Res Tracking option is enabled (default), the computational cells are decomposed into tetrahedrons
    (subtets) and the particles tracked through the subtets. This option provides a more robust tracking
    algorithm and improved variable interpolation. If High-Res Tracking is not enabled, particles are tracked through
    the computational cells directly. Flow variables that appear in particle equations either use cell-center values
    or a truncated Taylor series approximation for interpolation to the particle position.
    """

    fluent_name = "high-res-tracking-options"

    child_names = \
        ['always_use_face_centroid_with_periodics', 'interpolation',
         'boundary_layer_tracking', 'check_subtet_validity',
         'use_automatic_intersection_tolerance',
         'use_barycentric_intersection', 'particle_relocation',
         'remove_stuck_particles', 'use_barycentric_sampling',
         'use_quad_face_centroid']

    always_use_face_centroid_with_periodics: always_use_face_centroid_with_periodics = always_use_face_centroid_with_periodics
    """
    always_use_face_centroid_with_periodics child of high_res_tracking_options.
    """
    interpolation: interpolation = interpolation
    """
    interpolation child of high_res_tracking_options.
    """
    boundary_layer_tracking: boundary_layer_tracking = boundary_layer_tracking
    """
    boundary_layer_tracking child of high_res_tracking_options.
    """
    check_subtet_validity: check_subtet_validity = check_subtet_validity
    """
    check_subtet_validity child of high_res_tracking_options.
    """
    use_automatic_intersection_tolerance: use_automatic_intersection_tolerance = use_automatic_intersection_tolerance
    """
    use_automatic_intersection_tolerance child of high_res_tracking_options.
    """
    use_barycentric_intersection: use_barycentric_intersection = use_barycentric_intersection
    """
    use_barycentric_intersection child of high_res_tracking_options.
    """
    particle_relocation: particle_relocation = particle_relocation
    """
    particle_relocation child of high_res_tracking_options.
    """
    remove_stuck_particles: remove_stuck_particles = remove_stuck_particles
    """
    remove_stuck_particles child of high_res_tracking_options.
    """
    use_barycentric_sampling: use_barycentric_sampling = use_barycentric_sampling
    """
    use_barycentric_sampling child of high_res_tracking_options.
    """
    use_quad_face_centroid: use_quad_face_centroid = use_quad_face_centroid
    """
    use_quad_face_centroid child of high_res_tracking_options.
    """



"""A module used to provide abstract machine objects for queue system
interfaces.

This module provides two objects that help with interfacing python scripts with
the job scheduler environments:

  Machine
    This is used to represent a single machine allocated by the queue system
    and query details about it.

  MachineList
    This is used to load and query a queue system machine file.  Instances
    of this object hold a collection of Machine objects that are initialized
    when the machine file is loaded.
"""

from builtins import object
import copy


class Machine(object):
    """Provides an interface for a single machine allocated by a queue
    system."""

    def __init__(self, hostName, numberOfCores, queueName=None, coreList=None):
        """Constructs a machine from the information provided.

        Parameters
        ----------
        hostName : str
            Host name of the machine
        numberOfCores : int
            The number of cores allocated on the machine
        queueName : str
            Optionally specifies the queue the machine is executing in.
        coreList : list[int]
            Optionally provides the list of allocated core IDs.
        """
        self._hostName = hostName
        self._numberOfCores = numberOfCores
        self._queueName = queueName
        self._coreList = coreList

    def __repr__(self):
        """Returns a string representation for the machine."""
        return (
            "Hostname:"
            + self._hostName
            + ", Cores: "
            + str(self._numberOfCores)
            + ", Queue: "
            + self._queueName
        )

    @property
    def host_name(self):
        """Returns the hostname listed in the machine file."""
        return self._hostName

    @property
    def number_of_cores(self):
        """Returns the number of cores allocated on the machine."""
        return self._numberOfCores

    @number_of_cores.setter
    def number_of_cores(self, value):
        self._numberOfCores = value

    @property
    def queue_name(self):
        """Returns the name of the queue the machine is allocated in."""
        return self._queueName

    @property
    def core_list(self):
        """Returns a list of core IDs allocated on the machine."""
        return self._coreList


class MachineList(object):
    """Provides an interface to list of machines allocated by a queue
    system."""

    def __init__(self, machinesIn=[]):
        """Constructs and initializes an empty machine file object."""
        self._machines = []
        for machine in machinesIn:
            self._machines.append(machine)

    def __len__(self):
        return self.num_machines

    def __iter__(self):
        return self._machines.__iter__()

    def __getitem__(self, index):
        return self._machines[index]

    def __deepcopy__(self, memo):
        machineList = []
        for m in self.machines:
            machineList.append(m)
        return MachineList(copy.deepcopy(machineList, memo))

    def reset(self):
        """Resets the machine file data to the initial values."""
        self._machines = []

    def add(self, m):
        self._machines.append(m)

    def remove(self, m):
        self._machines.remove(m)

    def sort_by_core_count(self):
        """Sorts the machines by core count, reordering the existing data."""
        self._machines.sort(key=lambda machine: machine.number_of_cores, reverse=True)

    def sort_by_core_count_ascending(self):
        """Sorts the machines by core count, reordering the existing data."""
        self._machines.sort(key=lambda machine: machine.number_of_cores)

    def remove_empty_machines(self):
        """Removes all machines with 0 cores."""
        self._machines = [m for m in self._machines if m.number_of_cores > 0]

    def move_local_host_to_front(self):
        """Moves the local host machine to the front of the machine list,
        creating it if it does not exist."""
        import socket

        localHostName = socket.gethostname()
        localHostNameComponents = localHostName.split(".")
        localHostIndex = -1
        for im, m in enumerate(self._machines):
            # Check if hostName == localHostName, comparing as much of the name as possible
            hostNameComponents = m.host_name.split(".")
            imin = min(len(localHostNameComponents), len(hostNameComponents))
            if hostNameComponents[:imin] == localHostNameComponents[:imin]:
                localHostIndex = im
        # If the local host is in the list move it to the beginning
        if localHostIndex > -1:
            localMachine = self._machines.pop(localHostIndex)
            # Place the object in the front of the list
            self._machines.insert(0, localMachine)

    @property
    def machines(self):
        """Returns the entire list of machines."""
        return self._machines

    @property
    def num_machines(self):
        """Returns the total number of machines."""
        return len(self._machines)

    @property
    def number_of_cores(self):
        """Returns the total number of cores."""
        return sum([m.number_of_cores for m in self._machines])

    @property
    def max_cores(self):
        """Returns the maximum number of cores."""
        return max([m.number_of_cores for m in self._machines])

    @property
    def min_cores(self):
        """Returns the minimum number of cores."""
        return min([m.number_of_cores for m in self._machines])



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .option_11 import option
from .min_allowed import min_allowed
from .max_allowed import max_allowed
from .wall_zones import wall_zones
from .phase_26 import phase
class yplus_star(Group):
    """
    'yplus_star' child.
    """

    fluent_name = "yplus-star"

    child_names = \
        ['option', 'min_allowed', 'max_allowed', 'wall_zones', 'phase']

    option: option = option
    """
    option child of yplus_star.
    """
    min_allowed: min_allowed = min_allowed
    """
    min_allowed child of yplus_star.
    """
    max_allowed: max_allowed = max_allowed
    """
    max_allowed child of yplus_star.
    """
    wall_zones: wall_zones = wall_zones
    """
    wall_zones child of yplus_star.
    """
    phase: phase = phase
    """
    phase child of yplus_star.
    """



"""Provides a module for launching Fluent.

This module supports both starting Fluent locally and connecting to a
remote instance with gRPC.
"""
from enum import Enum
import json
import os
from pathlib import Path
import platform
import subprocess
import tempfile
import time
from typing import Any, Dict, Union
import warnings

from ansys.fluent.core.fluent_connection import _FluentConnection
from ansys.fluent.core.launcher.fluent_container import start_fluent_container
from ansys.fluent.core.scheduler import build_parallel_options, load_machines
from ansys.fluent.core.session import Session, _BaseSession, parse_server_info_file
from ansys.fluent.core.session_meshing import Meshing
from ansys.fluent.core.session_pure_meshing import PureMeshing
from ansys.fluent.core.session_solver import Solver
from ansys.fluent.core.session_solver_icing import SolverIcing
from ansys.fluent.core.utils.logging import LOG
import ansys.platform.instancemanagement as pypim

_THIS_DIR = os.path.dirname(__file__)
_OPTIONS_FILE = os.path.join(_THIS_DIR, "fluent_launcher_options.json")
FLUENT_VERSION = ["22.2"]
PIM_FLUENT_PRODUCT_VERSION = [FLUENT_VERSION[0].replace(".", "")]
FLUENT_EXE_PATH = []


class FluentVersion(Enum):
    """Contains the standard Ansys Fluent release."""

    version_22R2 = "22.2"
    version_23R1 = "23.1"

    @staticmethod
    def get_version(version: str) -> "FluentVersion":
        """Get the available versions based on the version in string format."""
        for v in FluentVersion:
            if version == v.value:
                return v
        else:
            raise RuntimeError(f"The passed version '{version}' does not exist.")


def set_fluent_path(fluent_exe_path: Union[str, Path]) -> None:
    """Set the Fluent installation path manually.

    This supersedes the Fluent path set in the environment variable.
    """
    if Path(fluent_exe_path).exists():
        FLUENT_EXE_PATH.append(str(fluent_exe_path))
    else:
        raise RuntimeError(
            f"The passed path '{fluent_exe_path}' does not contain a valid Fluent executable file."
        )


def set_ansys_version(version: Union[str, float, FluentVersion]) -> None:
    """Set the Fluent version manually.

    This method only works if the provided Fluent version is installed
    and the environment variables are updated properly. This supersedes
    the Fluent path set in the environment variable.
    """
    if type(version) in [float, str]:
        version = FluentVersion.get_version(str(version))
    if version in FluentVersion or str(version) in FluentVersion.value:
        FLUENT_VERSION[0] = version.value
        PIM_FLUENT_PRODUCT_VERSION[0] = FLUENT_VERSION[0].replace(".", "")


class LaunchModes(Enum):
    """Provides the standard Fluent launch modes."""

    # Tuple:   Name, Solver object type, Meshing flag, Launcher options
    MESHING_MODE = ("meshing", Meshing, True, [])
    PURE_MESHING_MODE = ("pure-meshing", PureMeshing, True, [])
    SOLVER = ("solver", Solver, False, [])
    SOLVER_ICING = ("solver-icing", SolverIcing, False, [("fluent_icing", True)])

    @staticmethod
    def get_mode(mode: str) -> "LaunchModes":
        """Returns the LaunchMode based on the mode in string format."""
        for m in LaunchModes:
            if mode == m.value[0]:
                return m
        else:
            raise RuntimeError(
                f"The passed mode '{mode}' matches none of the allowed modes."
            )


def get_fluent_path() -> Path:
    """Get the local Fluent installation path specified by PYFLUENT_FLUENT_ROOT
    or AWP_ROOTXXX environment variable.

    Returns
    -------
    str
        Local Fluent installation path.
    """
    if "PYFLUENT_FLUENT_ROOT" in os.environ:
        path = os.environ["PYFLUENT_FLUENT_ROOT"]
        return Path(path)
    else:
        path = os.environ["AWP_ROOT" + "".join(FLUENT_VERSION[0].split("."))]
        return Path(path) / "fluent"


def _get_fluent_exe_path():
    if FLUENT_EXE_PATH:
        exe_path = FLUENT_EXE_PATH[0]
    else:
        exe_path = get_fluent_path()
        if _is_windows():
            exe_path = exe_path / "ntbin" / "win64" / "fluent.exe"
        else:
            exe_path = exe_path / "bin" / "fluent"
    return str(exe_path)


def _get_server_info_filepath():
    server_info_dir = os.getenv("SERVER_INFO_DIR")
    dir_ = Path(server_info_dir) if server_info_dir else tempfile.gettempdir()
    fd, filepath = tempfile.mkstemp(suffix=".txt", prefix="serverinfo-", dir=str(dir_))
    os.close(fd)
    return filepath


def _get_subprocess_kwargs_for_fluent(env: Dict[str, Any]) -> Dict[str, Any]:
    kwargs: Dict[str, Any] = {}
    kwargs.update(stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    if _is_windows():
        kwargs.update(creationflags=subprocess.CREATE_NEW_PROCESS_GROUP)
    else:
        kwargs.update(shell=True, start_new_session=True)
    fluent_env = os.environ.copy()
    fluent_env.update({k: str(v) for k, v in env.items()})
    fluent_env["REMOTING_THROW_LAST_TUI_ERROR"] = "1"
    kwargs.update(env=fluent_env)
    return kwargs


def _build_fluent_launch_args_string(**kwargs) -> str:
    """Build Fluent's launch arguments string from keyword arguments.

    Returns
    -------
    str
        Fluent's launch arguments string.
    """
    all_options = None
    with open(_OPTIONS_FILE, encoding="utf-8") as fp:
        all_options = json.load(fp)
    launch_args_string = ""
    for k, v in all_options.items():
        argval = kwargs.get(k)
        default = v.get("default")
        if argval is None and v.get("fluent_required") is True:
            argval = default
        if argval is not None:
            allowed_values = v.get("allowed_values")
            if allowed_values and argval not in allowed_values:
                if default is not None:
                    old_argval = argval
                    argval = default
                    LOG.warning(
                        "Default value %s is chosen for %s as the passed "
                        "value  %s is outside allowed values %s.",
                        argval,
                        k,
                        old_argval,
                        allowed_values,
                    )
                else:
                    LOG.warning(
                        "%s = %s is discarded as it is outside " "allowed values %s.",
                        k,
                        argval,
                        allowed_values,
                    )
                    continue
            fluent_map = v.get("fluent_map")
            if fluent_map:
                if isinstance(argval, str):
                    json_key = argval
                else:
                    json_key = json.dumps(argval)
                argval = fluent_map[json_key]
            launch_args_string += v["fluent_format"].replace("{}", str(argval))
    addArgs = kwargs["additional_arguments"]
    if "-t" not in addArgs and "-cnf=" not in addArgs:
        mlist = load_machines(ncores=kwargs["processor_count"])
        launch_args_string += " " + build_parallel_options(mlist)
    return launch_args_string


def launch_remote_fluent(
    session_cls,
    start_transcript: bool,
    start_timeout: int = 100,
    product_version: str = None,
    cleanup_on_exit: bool = True,
    meshing_mode: bool = False,
    dimensionality: str = None,
):

    """Launch Fluent remotely using the PIM (Product Instance Management) API.

    When calling this method, you must ensure that you are in an
    environment where PyPIM is configured. You can use the :func:
    `pypim.is_configured <ansys.platform.instancemanagement.is_configured>`
    method to verify that PYPIM is configured.

    Parameters
    ----------
    session_cls: [_BaseSession, Session]
        Instance of the Session class
    start_transcript: bool
        Whether to start streaming the Fluent transcript in the client. The
        default is ``True``. You can stop and start the streaming of the
        Fluent transcript subsequently via method calls on the session object.
    start_timeout : int, optional
        Maximum allowable time in seconds for connecting to the Fluent
        server. The default is ``100``.
    product_version : str, optional
        Version of Fluent to use in the three-digit format (such as ``"212"``
        for 2021 R2). The default is ``None``, in which case the active version
        or latest installed version is used.
    cleanup_on_exit : bool, optional
        Whether to clean up and exit Fluent when Python exits or when garbage
        is collected for the Fluent Python instance. The default is ``True``.
    meshing_mode : bool, optional
        Whether to launch Fluent remotely in meshing mode. The default is
        ``False``.
    dimensionality : str, optional
        Number of dimensions for modeling. The default is ``None``, in which
        case ``"3s"`` is used. Options are ``"3d"`` and ``"2d"``.

    Returns
    -------
    ansys.fluent.core.session.Session
        Instance of the session.
    """
    pim = pypim.connect()
    instance = pim.create_instance(
        product_name="fluent-meshing"
        if meshing_mode
        else "fluent-2ddp"
        if dimensionality == "2d"
        else "fluent-3ddp",
        product_version=product_version,
    )
    instance.wait_for_ready()
    # nb pymapdl sets max msg len here:
    channel = instance.build_grpc_channel()
    return session_cls(
        fluent_connection=_FluentConnection(
            channel=channel,
            cleanup_on_exit=cleanup_on_exit,
            remote_instance=instance,
            start_timeout=start_timeout,
            start_transcript=start_transcript,
        )
    )


def _get_session_info(
    argvals, mode: Union[LaunchModes, str, None] = None, meshing_mode: bool = None
):
    """Updates the session information."""
    if mode is None:
        new_session = Session
    elif mode and meshing_mode:
        raise RuntimeError(
            "Please select either of the 2 ways of running ('mode' or 'meshing_mode')"
        )
    else:
        if type(mode) == str:
            mode = LaunchModes.get_mode(mode)
        new_session = mode.value[1]
        meshing_mode = mode.value[2]
        for k, v in mode.value[3]:
            argvals[k] = v

    return new_session, meshing_mode, argvals, mode


def _is_windows():
    """Check if the current operating system is windows."""
    return platform.system() == "Windows"


def _raise_exception_g_gu_in_windows_os(additional_arguments: str) -> None:
    """If -g or -gu is passed in Windows OS, the exception should be raised."""
    if _is_windows() and (
        ("-g" in additional_arguments) or ("-gu" in additional_arguments)
    ):
        raise ValueError("'-g' and '-gu' is not supported on windows platform.")


def _update_launch_string_wrt_gui_options(
    launch_string: str, show_gui: bool = None, additional_arguments: str = ""
) -> str:
    """Checks for all gui options in additional arguments and updates the
    launch string with hidden, if none of the options are met."""

    if (show_gui is False) or (
        show_gui is None and (os.getenv("PYFLUENT_SHOW_SERVER_GUI") != "1")
    ):
        if not {"-g", "-gu"} & set(additional_arguments.split()):
            launch_string += " -hidden"

    return launch_string


def _await_fluent_launch(
    server_info_filepath: str, start_timeout: int, sifile_last_mtime: float
):
    """Wait for successful fluent launch or raise an error."""
    while True:
        if Path(server_info_filepath).stat().st_mtime > sifile_last_mtime:
            time.sleep(1)
            LOG.info("Fluent process is successfully launched.")
            break
        if start_timeout == 0:
            raise RuntimeError("The launch process has been timed out.")
        time.sleep(1)
        start_timeout -= 1
        LOG.info(
            "Waiting for Fluent to launch...%02d seconds remaining",
            start_timeout,
        )


def _connect_to_running_server(argvals, server_info_filepath: str):
    """Connect to an already running session."""
    ip = argvals.get("ip", None)
    port = argvals.get("port", None)
    password = argvals.get("password", None)
    if ip and port:
        warnings.warn(
            "The server-info file was not parsed because ip and port were provided explicitly."
        )
    elif server_info_filepath:
        ip, port, password = parse_server_info_file(server_info_filepath)
    elif os.getenv("PYFLUENT_FLUENT_IP") and os.getenv("PYFLUENT_FLUENT_PORT"):
        ip = port = None
    else:
        raise RuntimeError(
            "Please provide either ip and port data or server-info file."
        )

    return ip, port, password


def _get_running_session_mode(
    fluent_connection: _FluentConnection, mode: LaunchModes = None
):
    """Get the mode of the running session if the mode has not been mentioned
    explicitly."""
    if mode:
        session_mode = mode
    else:
        try:
            session_mode = LaunchModes.get_mode(
                fluent_connection.get_current_fluent_mode()
            )
        except BaseException:
            raise RuntimeError("Fluent session password mismatch")
    return session_mode.value[1]


def _start_instance(start_instance: bool = None):
    """Sets up how to start an instance of fluent."""
    if not start_instance:
        start_instance = bool(
            int(
                os.getenv(
                    "PYFLUENT_START_INSTANCE", "0" if pypim.is_configured() else "1"
                )
            )
        )
    return start_instance


def _generate_launch_string(
    argvals,
    meshing_mode: bool,
    show_gui: bool,
    additional_arguments: str,
    server_info_filepath: str,
):
    """Generates the launch string to launch fluent."""
    exe_path = _get_fluent_exe_path()
    launch_string = exe_path
    launch_string += _build_fluent_launch_args_string(**argvals)
    if meshing_mode:
        launch_string += " -meshing"
    launch_string += f" {additional_arguments}"
    launch_string += f' -sifile="{server_info_filepath}"'
    launch_string += " -nm"
    launch_string = _update_launch_string_wrt_gui_options(
        launch_string, show_gui, additional_arguments
    )
    return launch_string


#   pylint: disable=unused-argument
def launch_fluent(
    version: str = None,
    precision: str = None,
    processor_count: int = None,
    journal_filename: str = None,
    start_timeout: int = 100,
    additional_arguments: str = "",
    env: Dict[str, Any] = None,
    start_instance: bool = None,
    ip: str = None,
    port: int = None,
    cleanup_on_exit: bool = True,
    start_transcript: bool = True,
    show_gui: bool = None,
    case_filepath: str = None,
    meshing_mode: bool = None,
    mode: Union[LaunchModes, str, None] = None,
    server_info_filepath: str = None,
    password: str = None,
    py: bool = None,
    cwd: str = None,
) -> Union[_BaseSession, Session]:
    """Launch Fluent locally in server mode or connect to a running Fluent
    server instance.

    Parameters
    ----------
    version : str, optional
        Dimensions for modeling. The default is ``None``, in which case ``"3d"``
        is used. Options are ``"3d"`` and ``"2d"``.
    precision : str, optional
        Floating point precision. The default is ``None``, in which case ``"double"``
        is used. Options are ``"double"`` and ``"single"``.
    processor_count : int, optional
        Number of processors. The default is ``None``, in which case ``1``
        processor is used.  In job scheduler environments the total number of
        allocated cores is clamped to this value.
    journal_filename : str, optional
        Name of the journal file to read. The default is ``None``.
    start_timeout : int, optional
        Maximum allowable time in seconds for connecting to the Fluent
        server. The default is ``100``.
    additional_arguments : str, optional
        Additional arguments to send to Fluent. The default is ``""``.
    env : dict[str, str], optional
        Mapping to modify environment variables in Fluent. The default
        is ``None``.
    start_instance : bool, optional
        Whether to connect to an existing Fluent instance at a specified IP
        address on a specified port. The default is ``None``, in which
        case a local instance of Fluent is started. When ``False``, use
        the next two parameters to specify the IP address and port. You
        can also use the environment variable ``PYFLUENT_START_INSTANCE=<0 or 1>``
        to set this parameter.
    ip : str, optional
        IP address for connecting to an existing Fluent instance. This parameter
        is used only when ``start_instance`` is ``False``. Otherwise, the
        IP address defaults to ``"127.0.0.1"``. You can also use the environment
        variable ``PYFLUENT_FLUENT_IP=<ip>`` to set this parameter.
    port : int, optional
        Port to listen on for an existing Fluent instance. This parameter is
        used only when ``start_instance`` is ``False``. You can use the
        environment variable ``PYFLUENT_FLUENT_PORT=<port>`` to set a default
        value.
    cleanup_on_exit : bool, optional
        Whether to shut down the connected Fluent session when PyFluent is
        exited or the ``exit()`` method is called on the session instance.
        The default is ``True``.
    start_transcript : bool, optional
        Whether to start streaming the Fluent transcript in the client. The
        default is ``True``. You can stop and start the streaming of the
        Fluent transcript subsequently via method calls on the session object.
    show_gui : bool, optional
        Whether to display the Fluent GUI when ``start_instance``
        is set to ''True``. The default is ``None`` so that explicit
        ``False`` settings can be detected. This is because you can use
        also use the environment variable ``PYFLUENT_SHOW_SERVER_GUI=<0 or 1>``
        to set this parameter. The ``show-gui`` parameter overrides the
        PYFLUENT_SHOW_SERVER_GUI environment variable. For example, if
        PYFLUENT_SHOW_SERVER_GUI is set to ``1`` and the ``show-gui``
        parameter is set to ``False``, the GUI is hidden.
    case_filepath : str, optional
        If provided, reads a fluent case file and sets the required settings
        in the fluent session
    meshing_mode : bool, optional
        Whether to launch Fluent in meshing mode. The default is ``None``,
        in which case Fluent is launched in meshing mode.
    mode : str, optional
        Launch mode of Fluent to point to a specific session type.
        The default value is ``None``. Options are ``"meshing"``,
        ``"pure-meshing"`` and ``"solver"``.
    server_info_filepath: str
        Path to server-info file written out by Fluent server. The default is ``None``.
    password : str, optional
            Password to connect to existing Fluent instance.
    py : bool, optional
        Passes ``"-py"`` as an additional_argument to launch fluent in python mode.
        The default is ``None``.

    cwd: str, Optional
        Path to specify current working directory to launch fluent from the defined directory as
        current working directory.

    Returns
    -------
    ansys.fluent.session.Session
        Fluent session.

    Notes
    -----
    In job scheduler environments such as SLURM, LSF, PBS, etc... the allocated
    machines and core counts are queried from the scheduler environment and
    passed to Fluent.
    """
    argvals = locals()

    new_session, meshing_mode, argvals, mode = _get_session_info(
        argvals, mode, meshing_mode
    )
    _raise_exception_g_gu_in_windows_os(additional_arguments)
    if _start_instance(start_instance):
        server_info_filepath = _get_server_info_filepath()
        launch_string = _generate_launch_string(
            argvals, meshing_mode, show_gui, additional_arguments, server_info_filepath
        )

        try:
            LOG.info("Launching Fluent with cmd: %s", launch_string)
            sifile_last_mtime = Path(server_info_filepath).stat().st_mtime
            if env is None:
                env = {}
            if mode != LaunchModes.SOLVER_ICING:
                env["APP_LAUNCHED_FROM_CLIENT"] = "1"  # disables flserver datamodel
            kwargs = _get_subprocess_kwargs_for_fluent(env)
            if cwd:
                kwargs.update(cwd=cwd)
            subprocess.Popen(launch_string, **kwargs)

            _await_fluent_launch(server_info_filepath, start_timeout, sifile_last_mtime)

            return new_session.create_from_server_info_file(
                server_info_filepath, cleanup_on_exit, start_transcript
            )
        finally:
            server_info_file = Path(server_info_filepath)
            if server_info_file.exists():
                server_info_file.unlink()
    else:
        if pypim.is_configured():
            LOG.info(
                "Starting Fluent remotely. The startup configuration will be ignored."
            )
            return launch_remote_fluent(
                session_cls=new_session,
                start_timeout=start_timeout,
                start_transcript=start_transcript,
                product_version=PIM_FLUENT_PRODUCT_VERSION[0],
                cleanup_on_exit=cleanup_on_exit,
                meshing_mode=meshing_mode,
                dimensionality=version,
            )
        import ansys.fluent.core as pyfluent

        if pyfluent.BUILDING_GALLERY or os.getenv("PYFLUENT_LAUNCH_CONTAINER") == "1":
            args = _build_fluent_launch_args_string(**argvals).split()
            if meshing_mode:
                args.append(" -meshing")
            # Assumes the container OS will be able to create the
            # EXAMPLES_PATH of host OS. With the Fluent docker
            # container, the following currently works only in linux.
            port, password = start_fluent_container(
                pyfluent.EXAMPLES_PATH, pyfluent.EXAMPLES_PATH, args
            )
            return new_session(
                fluent_connection=_FluentConnection(
                    start_timeout=start_timeout,
                    port=port,
                    password=password,
                    cleanup_on_exit=cleanup_on_exit,
                    start_transcript=start_transcript,
                )
            )
        else:
            ip, port, password = _connect_to_running_server(
                argvals, server_info_filepath
            )
            fluent_connection = _FluentConnection(
                ip=ip,
                port=port,
                password=password,
                cleanup_on_exit=cleanup_on_exit,
                start_transcript=start_transcript,
            )
            new_session = _get_running_session_mode(fluent_connection, mode)
            return new_session(fluent_connection=fluent_connection)



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .relative_permeability import relative_permeability
from .capillary_pressure_as_diffusion import capillary_pressure_as_diffusion
class porous_media(Group):
    """
    Multiphase miscellaneous porous media numerics menu.
    """

    fluent_name = "porous-media"

    child_names = \
        ['relative_permeability', 'capillary_pressure_as_diffusion']

    relative_permeability: relative_permeability = relative_permeability
    """
    relative_permeability child of porous_media.
    """
    capillary_pressure_as_diffusion: capillary_pressure_as_diffusion = capillary_pressure_as_diffusion
    """
    capillary_pressure_as_diffusion child of porous_media.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .viscous_heating import viscous_heating
from .low_pressure_slip import low_pressure_slip
from .curvature_correction import curvature_correction
from .corner_flow_correction import corner_flow_correction
from .production_kato_launder import production_kato_launder
from .production_limiter import production_limiter
class options(Group):
    """
    'options' child.
    """

    fluent_name = "options"

    child_names = \
        ['viscous_heating', 'low_pressure_slip', 'curvature_correction',
         'corner_flow_correction', 'production_kato_launder',
         'production_limiter']

    viscous_heating: viscous_heating = viscous_heating
    """
    viscous_heating child of options.
    """
    low_pressure_slip: low_pressure_slip = low_pressure_slip
    """
    low_pressure_slip child of options.
    """
    curvature_correction: curvature_correction = curvature_correction
    """
    curvature_correction child of options.
    """
    corner_flow_correction: corner_flow_correction = corner_flow_correction
    """
    corner_flow_correction child of options.
    """
    production_kato_launder: production_kato_launder = production_kato_launder
    """
    production_kato_launder child of options.
    """
    production_limiter: production_limiter = production_limiter
    """
    production_limiter child of options.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#
# pylint: disable=line-too-long

from ansys.fluent.core.services.datamodel_se import (
    PyMenu,
    PyParameter,
    PyTextual,
    PyNumerical,
    PyDictionary,
    PyNamedObjectContainer,
    PyCommand
)


class Root(PyMenu):
    """
    Singleton Root.
    """
    def __init__(self, service, rules, path):
        self.TaskObject = self.__class__.TaskObject(service, rules, path + [("TaskObject", "")])
        self.Workflow = self.__class__.Workflow(service, rules, path + [("Workflow", "")])
        self.CreateCompositeTask = self.__class__.CreateCompositeTask(service, rules, "CreateCompositeTask", path)
        self.CreateNewWorkflow = self.__class__.CreateNewWorkflow(service, rules, "CreateNewWorkflow", path)
        self.DeleteTasks = self.__class__.DeleteTasks(service, rules, "DeleteTasks", path)
        self.InitializeWorkflow = self.__class__.InitializeWorkflow(service, rules, "InitializeWorkflow", path)
        self.InsertNewTask = self.__class__.InsertNewTask(service, rules, "InsertNewTask", path)
        self.LoadState = self.__class__.LoadState(service, rules, "LoadState", path)
        self.LoadWorkflow = self.__class__.LoadWorkflow(service, rules, "LoadWorkflow", path)
        self.ResetWorkflow = self.__class__.ResetWorkflow(service, rules, "ResetWorkflow", path)
        self.SaveWorkflow = self.__class__.SaveWorkflow(service, rules, "SaveWorkflow", path)
        super().__init__(service, rules, path)

    class TaskObject(PyNamedObjectContainer):
        """
        .
        """
        class _TaskObject(PyMenu):
            """
            Singleton _TaskObject.
            """
            def __init__(self, service, rules, path):
                self.Arguments = self.__class__.Arguments(service, rules, path + [("Arguments", "")])
                self.CommandName = self.__class__.CommandName(service, rules, path + [("CommandName", "")])
                self.Errors = self.__class__.Errors(service, rules, path + [("Errors", "")])
                self.InactiveTaskList = self.__class__.InactiveTaskList(service, rules, path + [("InactiveTaskList", "")])
                self.ObjectPath = self.__class__.ObjectPath(service, rules, path + [("ObjectPath", "")])
                self.State = self.__class__.State(service, rules, path + [("State", "")])
                self.TaskList = self.__class__.TaskList(service, rules, path + [("TaskList", "")])
                self.TaskType = self.__class__.TaskType(service, rules, path + [("TaskType", "")])
                self.Warnings = self.__class__.Warnings(service, rules, path + [("Warnings", "")])
                self._name_ = self.__class__._name_(service, rules, path + [("_name_", "")])
                self.AddChildAndUpdate = self.__class__.AddChildAndUpdate(service, rules, "AddChildAndUpdate", path)
                self.AddChildToTask = self.__class__.AddChildToTask(service, rules, "AddChildToTask", path)
                self.Execute = self.__class__.Execute(service, rules, "Execute", path)
                self.ExecuteUpstreamNonExecutedAndThisTask = self.__class__.ExecuteUpstreamNonExecutedAndThisTask(service, rules, "ExecuteUpstreamNonExecutedAndThisTask", path)
                self.ForceUptoDate = self.__class__.ForceUptoDate(service, rules, "ForceUptoDate", path)
                self.GetNextPossibleTasks = self.__class__.GetNextPossibleTasks(service, rules, "GetNextPossibleTasks", path)
                self.InsertCompositeChildTask = self.__class__.InsertCompositeChildTask(service, rules, "InsertCompositeChildTask", path)
                self.InsertCompoundChildTask = self.__class__.InsertCompoundChildTask(service, rules, "InsertCompoundChildTask", path)
                self.InsertNextTask = self.__class__.InsertNextTask(service, rules, "InsertNextTask", path)
                self.Rename = self.__class__.Rename(service, rules, "Rename", path)
                self.Revert = self.__class__.Revert(service, rules, "Revert", path)
                self.SetAsCurrent = self.__class__.SetAsCurrent(service, rules, "SetAsCurrent", path)
                self.UpdateChildTasks = self.__class__.UpdateChildTasks(service, rules, "UpdateChildTasks", path)
                super().__init__(service, rules, path)

            class Arguments(PyDictionary):
                """
                Parameter Arguments of value type Dict[str, Any].
                """
                pass

            class CommandName(PyTextual):
                """
                Parameter CommandName of value type str.
                """
                pass

            class Errors(PyTextual):
                """
                Parameter Errors of value type List[str].
                """
                pass

            class InactiveTaskList(PyTextual):
                """
                Parameter InactiveTaskList of value type List[str].
                """
                pass

            class ObjectPath(PyTextual):
                """
                Parameter ObjectPath of value type str.
                """
                pass

            class State(PyTextual):
                """
                Parameter State of value type str.
                """
                pass

            class TaskList(PyTextual):
                """
                Parameter TaskList of value type List[str].
                """
                pass

            class TaskType(PyTextual):
                """
                Parameter TaskType of value type str.
                """
                pass

            class Warnings(PyTextual):
                """
                Parameter Warnings of value type List[str].
                """
                pass

            class _name_(PyTextual):
                """
                Parameter _name_ of value type str.
                """
                pass

            class AddChildAndUpdate(PyCommand):
                """
                Command AddChildAndUpdate.


                Returns
                -------
                bool
                """
                pass

            class AddChildToTask(PyCommand):
                """
                Command AddChildToTask.


                Returns
                -------
                bool
                """
                pass

            class Execute(PyCommand):
                """
                Command Execute.

                Parameters
                ----------
                Force : bool

                Returns
                -------
                bool
                """
                pass

            class ExecuteUpstreamNonExecutedAndThisTask(PyCommand):
                """
                Command ExecuteUpstreamNonExecutedAndThisTask.


                Returns
                -------
                bool
                """
                pass

            class ForceUptoDate(PyCommand):
                """
                Command ForceUptoDate.


                Returns
                -------
                bool
                """
                pass

            class GetNextPossibleTasks(PyCommand):
                """
                Command GetNextPossibleTasks.


                Returns
                -------
                bool
                """
                pass

            class InsertCompositeChildTask(PyCommand):
                """
                Command InsertCompositeChildTask.

                Parameters
                ----------
                CommandName : str

                Returns
                -------
                bool
                """
                pass

            class InsertCompoundChildTask(PyCommand):
                """
                Command InsertCompoundChildTask.


                Returns
                -------
                bool
                """
                pass

            class InsertNextTask(PyCommand):
                """
                Command InsertNextTask.

                Parameters
                ----------
                CommandName : str
                Select : bool

                Returns
                -------
                bool
                """
                pass

            class Rename(PyCommand):
                """
                Command Rename.

                Parameters
                ----------
                NewName : str

                Returns
                -------
                bool
                """
                pass

            class Revert(PyCommand):
                """
                Command Revert.


                Returns
                -------
                bool
                """
                pass

            class SetAsCurrent(PyCommand):
                """
                Command SetAsCurrent.


                Returns
                -------
                bool
                """
                pass

            class UpdateChildTasks(PyCommand):
                """
                Command UpdateChildTasks.

                Parameters
                ----------
                SetupTypeChanged : bool

                Returns
                -------
                bool
                """
                pass

        def __getitem__(self, key: str) -> _TaskObject:
            return super().__getitem__(key)

    class Workflow(PyMenu):
        """
        Singleton Workflow.
        """
        def __init__(self, service, rules, path):
            self.CurrentTask = self.__class__.CurrentTask(service, rules, path + [("CurrentTask", "")])
            self.TaskList = self.__class__.TaskList(service, rules, path + [("TaskList", "")])
            super().__init__(service, rules, path)

        class CurrentTask(PyTextual):
            """
            Parameter CurrentTask of value type str.
            """
            pass

        class TaskList(PyTextual):
            """
            Parameter TaskList of value type List[str].
            """
            pass

    class CreateCompositeTask(PyCommand):
        """
        Command CreateCompositeTask.

        Parameters
        ----------
        ListOfTasks : List[str]

        Returns
        -------
        bool
        """
        pass

    class CreateNewWorkflow(PyCommand):
        """
        Command CreateNewWorkflow.


        Returns
        -------
        bool
        """
        pass

    class DeleteTasks(PyCommand):
        """
        Command DeleteTasks.

        Parameters
        ----------
        ListOfTasks : List[str]

        Returns
        -------
        bool
        """
        pass

    class InitializeWorkflow(PyCommand):
        """
        Command InitializeWorkflow.

        Parameters
        ----------
        WorkflowType : str

        Returns
        -------
        bool
        """
        pass

    class InsertNewTask(PyCommand):
        """
        Command InsertNewTask.

        Parameters
        ----------
        CommandName : str

        Returns
        -------
        bool
        """
        pass

    class LoadState(PyCommand):
        """
        Command LoadState.

        Parameters
        ----------
        ListOfRoots : List[str]

        Returns
        -------
        bool
        """
        pass

    class LoadWorkflow(PyCommand):
        """
        Command LoadWorkflow.

        Parameters
        ----------
        FilePath : str

        Returns
        -------
        bool
        """
        pass

    class ResetWorkflow(PyCommand):
        """
        Command ResetWorkflow.


        Returns
        -------
        bool
        """
        pass

    class SaveWorkflow(PyCommand):
        """
        Command SaveWorkflow.

        Parameters
        ----------
        FilePath : str

        Returns
        -------
        bool
        """
        pass




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .first_to_second_order_blending_dbns import first_to_second_order_blending_dbns
class numerics_dbns(Group):
    """
    'numerics_dbns' child.
    """

    fluent_name = "numerics-dbns"

    child_names = \
        ['first_to_second_order_blending_dbns']

    first_to_second_order_blending_dbns: first_to_second_order_blending_dbns = first_to_second_order_blending_dbns
    """
    first_to_second_order_blending_dbns child of numerics_dbns.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .fluid import fluid
from .solid import solid
from .mixture import mixture
from .inert_particle import inert_particle
from .droplet_particle import droplet_particle
from .combusting_particle import combusting_particle
from .particle_mixture import particle_mixture
from .list_materials import list_materials
from .copy_database_material_by_name import copy_database_material_by_name
from .copy_database_material_by_formula import copy_database_material_by_formula
class materials(Group):
    """
    'materials' child.
    """

    fluent_name = "materials"

    child_names = \
        ['fluid', 'solid', 'mixture', 'inert_particle', 'droplet_particle',
         'combusting_particle', 'particle_mixture']

    fluid: fluid = fluid
    """
    fluid child of materials.
    """
    solid: solid = solid
    """
    solid child of materials.
    """
    mixture: mixture = mixture
    """
    mixture child of materials.
    """
    inert_particle: inert_particle = inert_particle
    """
    inert_particle child of materials.
    """
    droplet_particle: droplet_particle = droplet_particle
    """
    droplet_particle child of materials.
    """
    combusting_particle: combusting_particle = combusting_particle
    """
    combusting_particle child of materials.
    """
    particle_mixture: particle_mixture = particle_mixture
    """
    particle_mixture child of materials.
    """
    command_names = \
        ['list_materials', 'copy_database_material_by_name',
         'copy_database_material_by_formula']

    list_materials: list_materials = list_materials
    """
    list_materials command of materials.
    """
    copy_database_material_by_name: copy_database_material_by_name = copy_database_material_by_name
    """
    copy_database_material_by_name command of materials.
    """
    copy_database_material_by_formula: copy_database_material_by_formula = copy_database_material_by_formula
    """
    copy_database_material_by_formula command of materials.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .minimum_val import minimum_val
from .maximum_val import maximum_val
from .division_val import division_val
class histogram_parameters(Group):
    """
    Enter the parameter menu for the histogram.
    """

    fluent_name = "histogram-parameters"

    child_names = \
        ['minimum_val', 'maximum_val', 'division_val']

    minimum_val: minimum_val = minimum_val
    """
    minimum_val child of histogram_parameters.
    """
    maximum_val: maximum_val = maximum_val
    """
    maximum_val child of histogram_parameters.
    """
    division_val: division_val = division_val
    """
    division_val child of histogram_parameters.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .moment_child import moment_child

class moment(NamedObject[moment_child], _CreatableNamedObjectMixin[moment_child]):
    """
    'moment' child.
    """

    fluent_name = "moment"

    child_object_type: moment_child = moment_child
    """
    child_object_type of moment.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .name_1 import name
from .field import field
from .vector_field import vector_field
from .surfaces_list import surfaces_list
from .scale_1 import scale
from .style import style
from .skip import skip
from .vector_opt import vector_opt
from .range_option import range_option
from .color_map import color_map
from .draw_mesh import draw_mesh
from .mesh_object import mesh_object
from .display_state_name import display_state_name
from .physics import physics
from .geometry_3 import geometry
from .surfaces import surfaces
from .display_2 import display
class vector_child(Group):
    """
    'child_object_type' of vector.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['name', 'field', 'vector_field', 'surfaces_list', 'scale', 'style',
         'skip', 'vector_opt', 'range_option', 'color_map', 'draw_mesh',
         'mesh_object', 'display_state_name', 'physics', 'geometry',
         'surfaces']

    name: name = name
    """
    name child of vector_child.
    """
    field: field = field
    """
    field child of vector_child.
    """
    vector_field: vector_field = vector_field
    """
    vector_field child of vector_child.
    """
    surfaces_list: surfaces_list = surfaces_list
    """
    surfaces_list child of vector_child.
    """
    scale: scale = scale
    """
    scale child of vector_child.
    """
    style: style = style
    """
    style child of vector_child.
    """
    skip: skip = skip
    """
    skip child of vector_child.
    """
    vector_opt: vector_opt = vector_opt
    """
    vector_opt child of vector_child.
    """
    range_option: range_option = range_option
    """
    range_option child of vector_child.
    """
    color_map: color_map = color_map
    """
    color_map child of vector_child.
    """
    draw_mesh: draw_mesh = draw_mesh
    """
    draw_mesh child of vector_child.
    """
    mesh_object: mesh_object = mesh_object
    """
    mesh_object child of vector_child.
    """
    display_state_name: display_state_name = display_state_name
    """
    display_state_name child of vector_child.
    """
    physics: physics = physics
    """
    physics child of vector_child.
    """
    geometry: geometry = geometry
    """
    geometry child of vector_child.
    """
    surfaces: surfaces = surfaces
    """
    surfaces child of vector_child.
    """
    command_names = \
        ['display']

    display: display = display
    """
    display command of vector_child.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .pressure_velocity_coupling_controls import pressure_velocity_coupling_controls
from .pressure_velocity_coupling_method import pressure_velocity_coupling_method
from .gradient_controls import gradient_controls
from .specify_gradient_method import specify_gradient_method
class methods(Group):
    """
    'methods' child.
    """

    fluent_name = "methods"

    child_names = \
        ['pressure_velocity_coupling_controls',
         'pressure_velocity_coupling_method', 'gradient_controls',
         'specify_gradient_method']

    pressure_velocity_coupling_controls: pressure_velocity_coupling_controls = pressure_velocity_coupling_controls
    """
    pressure_velocity_coupling_controls child of methods.
    """
    pressure_velocity_coupling_method: pressure_velocity_coupling_method = pressure_velocity_coupling_method
    """
    pressure_velocity_coupling_method child of methods.
    """
    gradient_controls: gradient_controls = gradient_controls
    """
    gradient_controls child of methods.
    """
    specify_gradient_method: specify_gradient_method = specify_gradient_method
    """
    specify_gradient_method child of methods.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .use_layering import use_layering
from .base_face_zone_for_partitioning import base_face_zone_for_partitioning
class layering(Group):
    """
    Use layering for partitioning.
    """

    fluent_name = "layering"

    child_names = \
        ['use_layering', 'base_face_zone_for_partitioning']

    use_layering: use_layering = use_layering
    """
    use_layering child of layering.
    """
    base_face_zone_for_partitioning: base_face_zone_for_partitioning = base_face_zone_for_partitioning
    """
    base_face_zone_for_partitioning child of layering.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .lines import lines
from .markers import markers
class curves(Group):
    """
    'curves' child.
    """

    fluent_name = "curves"

    child_names = \
        ['lines', 'markers']

    lines: lines = lines
    """
    lines child of curves.
    """
    markers: markers = markers
    """
    markers child of curves.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .list import list
from .use_active import use_active
from .restore_state import restore_state
from .copy import copy
from .read_1 import read
from .write_1 import write
from .display_states_child import display_states_child

class display_states(NamedObject[display_states_child], _CreatableNamedObjectMixin[display_states_child]):
    """
    'display_states' child.
    """

    fluent_name = "display-states"

    command_names = \
        ['list', 'use_active', 'restore_state', 'copy', 'read', 'write']

    list: list = list
    """
    list command of display_states.
    """
    use_active: use_active = use_active
    """
    use_active command of display_states.
    """
    restore_state: restore_state = restore_state
    """
    restore_state command of display_states.
    """
    copy: copy = copy
    """
    copy command of display_states.
    """
    read: read = read
    """
    read command of display_states.
    """
    write: write = write
    """
    write command of display_states.
    """
    child_object_type: display_states_child = display_states_child
    """
    child_object_type of display_states.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .heat_exchanger import heat_exchanger
from .fluid_zone import fluid_zone
from .boundary_zone import boundary_zone
from .report_type import report_type
from .write_to_file import write_to_file
from .file_name_1 import file_name
from .append_file import append_file
from .overwrite import overwrite
class inlet_temperature(Command):
    """
    'inlet_temperature' command.
    
    Parameters
    ----------
        heat_exchanger : str
            'heat_exchanger' child.
        fluid_zone : str
            'fluid_zone' child.
        boundary_zone : str
            'boundary_zone' child.
        report_type : str
            'report_type' child.
        write_to_file : bool
            'write_to_file' child.
        file_name : str
            'file_name' child.
        append_file : bool
            'append_file' child.
        overwrite : bool
            'overwrite' child.
    
    """

    fluent_name = "inlet-temperature"

    argument_names = \
        ['heat_exchanger', 'fluid_zone', 'boundary_zone', 'report_type',
         'write_to_file', 'file_name', 'append_file', 'overwrite']

    heat_exchanger: heat_exchanger = heat_exchanger
    """
    heat_exchanger argument of inlet_temperature.
    """
    fluid_zone: fluid_zone = fluid_zone
    """
    fluid_zone argument of inlet_temperature.
    """
    boundary_zone: boundary_zone = boundary_zone
    """
    boundary_zone argument of inlet_temperature.
    """
    report_type: report_type = report_type
    """
    report_type argument of inlet_temperature.
    """
    write_to_file: write_to_file = write_to_file
    """
    write_to_file argument of inlet_temperature.
    """
    file_name: file_name = file_name
    """
    file_name argument of inlet_temperature.
    """
    append_file: append_file = append_file
    """
    append_file argument of inlet_temperature.
    """
    overwrite: overwrite = overwrite
    """
    overwrite argument of inlet_temperature.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .display import display
from .pathlines_child import pathlines_child

class pathlines(NamedObject[pathlines_child], _CreatableNamedObjectMixin[pathlines_child]):
    """
    'pathlines' child.
    """

    fluent_name = "pathlines"

    command_names = \
        ['display']

    display: display = display
    """
    display command of pathlines.
    """
    child_object_type: pathlines_child = pathlines_child
    """
    child_object_type of pathlines.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .set_defaults_child import set_defaults_child

class set_defaults(NamedObject[set_defaults_child], _NonCreatableNamedObjectMixin[set_defaults_child]):
    """
    'set_defaults' child.
    """

    fluent_name = "set-defaults"

    child_object_type: set_defaults_child = set_defaults_child
    """
    child_object_type of set_defaults.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class create_particles_every_particle_step(Boolean):
    """
    Create particles every particle time step (else every fluid time step)?.
    """

    fluent_name = "create-particles-every-particle-step?"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .compute_statistics import compute_statistics
from .statistics_level import statistics_level
class miscellaneous(Group):
    """
    'miscellaneous' child.
    """

    fluent_name = "miscellaneous"

    child_names = \
        ['compute_statistics', 'statistics_level']

    compute_statistics: compute_statistics = compute_statistics
    """
    compute_statistics child of miscellaneous.
    """
    statistics_level: statistics_level = statistics_level
    """
    statistics_level child of miscellaneous.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .recirculation_inlet_child import recirculation_inlet_child

class recirculation_inlet(NamedObject[recirculation_inlet_child], _NonCreatableNamedObjectMixin[recirculation_inlet_child]):
    """
    'recirculation_inlet' child.
    """

    fluent_name = "recirculation-inlet"

    child_object_type: recirculation_inlet_child = recirculation_inlet_child
    """
    child_object_type of recirculation_inlet.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class extrapolation_method(String):
    """
    Solution extrapolation method for cells changing status from contact to non-contact.
    """

    fluent_name = "extrapolation-method"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .option_9 import option
from .lewis_number import lewis_number
from .mass_diffusivity import mass_diffusivity
from .species_diffusivity import species_diffusivity
from .multicomponent import multicomponent
from .user_defined_function import user_defined_function
class mass_diffusivity(Group):
    """
    'mass_diffusivity' child.
    """

    fluent_name = "mass-diffusivity"

    child_names = \
        ['option', 'lewis_number', 'mass_diffusivity', 'species_diffusivity',
         'multicomponent', 'user_defined_function']

    option: option = option
    """
    option child of mass_diffusivity.
    """
    lewis_number: lewis_number = lewis_number
    """
    lewis_number child of mass_diffusivity.
    """
    mass_diffusivity: mass_diffusivity = mass_diffusivity
    """
    mass_diffusivity child of mass_diffusivity.
    """
    species_diffusivity: species_diffusivity = species_diffusivity
    """
    species_diffusivity child of mass_diffusivity.
    """
    multicomponent: multicomponent = multicomponent
    """
    multicomponent child of mass_diffusivity.
    """
    user_defined_function: user_defined_function = user_defined_function
    """
    user_defined_function child of mass_diffusivity.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .file_type import file_type
from .file_name_1 import file_name
from .pdf_file_name import pdf_file_name
from .lightweight_setup import lightweight_setup
class read(Command):
    """
    'read' command.
    
    Parameters
    ----------
        file_type : str
            'file_type' child.
        file_name : str
            'file_name' child.
        pdf_file_name : str
            'pdf_file_name' child.
        lightweight_setup : bool
            'lightweight_setup' child.
    
    """

    fluent_name = "read"

    argument_names = \
        ['file_type', 'file_name', 'pdf_file_name', 'lightweight_setup']

    file_type: file_type = file_type
    """
    file_type argument of read.
    """
    file_name: file_name = file_name
    """
    file_name argument of read.
    """
    pdf_file_name: pdf_file_name = pdf_file_name
    """
    pdf_file_name argument of read.
    """
    lightweight_setup: lightweight_setup = lightweight_setup
    """
    lightweight_setup argument of read.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .mesh_1 import mesh
from .contour import contour
from .vector import vector
from .pathlines import pathlines
from .particle_tracks import particle_tracks
from .lic import lic
from .views import views
class graphics(Group):
    """
    'graphics' child.
    """

    fluent_name = "graphics"

    child_names = \
        ['mesh', 'contour', 'vector', 'pathlines', 'particle_tracks', 'lic',
         'views']

    mesh: mesh = mesh
    """
    mesh child of graphics.
    """
    contour: contour = contour
    """
    contour child of graphics.
    """
    vector: vector = vector
    """
    vector child of graphics.
    """
    pathlines: pathlines = pathlines
    """
    pathlines child of graphics.
    """
    particle_tracks: particle_tracks = particle_tracks
    """
    particle_tracks child of graphics.
    """
    lic: lic = lic
    """
    lic child of graphics.
    """
    views: views = views
    """
    views child of graphics.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .draw_mesh import draw_mesh
from .filled import filled
from .marker import marker
from .marker_symbol import marker_symbol
from .marker_size import marker_size
from .wireframe import wireframe
from .color import color
class display_options(Group):
    """
    'display_options' child.
    """

    fluent_name = "display-options"

    child_names = \
        ['draw_mesh', 'filled', 'marker', 'marker_symbol', 'marker_size',
         'wireframe', 'color']

    draw_mesh: draw_mesh = draw_mesh
    """
    draw_mesh child of display_options.
    """
    filled: filled = filled
    """
    filled child of display_options.
    """
    marker: marker = marker
    """
    marker child of display_options.
    """
    marker_symbol: marker_symbol = marker_symbol
    """
    marker_symbol child of display_options.
    """
    marker_size: marker_size = marker_size
    """
    marker_size child of display_options.
    """
    wireframe: wireframe = wireframe
    """
    wireframe child of display_options.
    """
    color: color = color
    """
    color child of display_options.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .enable_solid_time_step import enable_solid_time_step
from .choose_auto_time_stepping import choose_auto_time_stepping
from .time_step_size_1 import time_step_size
class solid_time_step_size(Group):
    """
    'solid_time_step_size' child.
    """

    fluent_name = "solid-time-step-size"

    child_names = \
        ['enable_solid_time_step', 'choose_auto_time_stepping',
         'time_step_size']

    enable_solid_time_step: enable_solid_time_step = enable_solid_time_step
    """
    enable_solid_time_step child of solid_time_step_size.
    """
    choose_auto_time_stepping: choose_auto_time_stepping = choose_auto_time_stepping
    """
    choose_auto_time_stepping child of solid_time_step_size.
    """
    time_step_size: time_step_size = time_step_size
    """
    time_step_size child of solid_time_step_size.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .thickness import thickness
from .material import material
from .qdot import qdot
class shell_conduction_child(Group):
    """
    'child_object_type' of shell_conduction.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['thickness', 'material', 'qdot']

    thickness: thickness = thickness
    """
    thickness child of shell_conduction_child.
    """
    material: material = material
    """
    material child of shell_conduction_child.
    """
    qdot: qdot = qdot
    """
    qdot child of shell_conduction_child.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class density_ratio_cutoff(Real):
    """
    Density ratio cut-off.
    """

    fluent_name = "density-ratio-cutoff"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class two_resistance_boiling_framework(Boolean):
    """
    Allow generalized two-resistance framework for boiling model.
    """

    fluent_name = "two-resistance-boiling-framework?"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class active_wetsteam_zone(Boolean):
    """
    'active_wetsteam_zone' child.
    """

    fluent_name = "active-wetsteam-zone?"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .plot_sample import plot_sample
from .write_sample import write_sample
class plot_write_sample(Group):
    """
    'plot_write_sample' child.
    """

    fluent_name = "plot-write-sample"

    command_names = \
        ['plot_sample', 'write_sample']

    plot_sample: plot_sample = plot_sample
    """
    plot_sample command of plot_write_sample.
    """
    write_sample: write_sample = write_sample
    """
    write_sample command of plot_write_sample.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .enabled_3 import enabled
from .set_5 import set
class conjugate_heat_transfer(Group):
    """
    'conjugate_heat_transfer' child.
    """

    fluent_name = "conjugate-heat-transfer"

    child_names = \
        ['enabled', 'set']

    enabled: enabled = enabled
    """
    enabled child of conjugate_heat_transfer.
    """
    set: set = set
    """
    set child of conjugate_heat_transfer.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .name_1 import name
from .animate_on import animate_on
from .frequency_1 import frequency
from .flow_time_frequency import flow_time_frequency
from .frequency_of import frequency_of
from .storage_type import storage_type
from .storage_dir import storage_dir
from .window_id import window_id
from .view import view
from .use_raytracing import use_raytracing
from .display_2 import display
class solution_animations_child(Group):
    """
    'child_object_type' of solution_animations.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['name', 'animate_on', 'frequency', 'flow_time_frequency',
         'frequency_of', 'storage_type', 'storage_dir', 'window_id', 'view',
         'use_raytracing']

    name: name = name
    """
    name child of solution_animations_child.
    """
    animate_on: animate_on = animate_on
    """
    animate_on child of solution_animations_child.
    """
    frequency: frequency = frequency
    """
    frequency child of solution_animations_child.
    """
    flow_time_frequency: flow_time_frequency = flow_time_frequency
    """
    flow_time_frequency child of solution_animations_child.
    """
    frequency_of: frequency_of = frequency_of
    """
    frequency_of child of solution_animations_child.
    """
    storage_type: storage_type = storage_type
    """
    storage_type child of solution_animations_child.
    """
    storage_dir: storage_dir = storage_dir
    """
    storage_dir child of solution_animations_child.
    """
    window_id: window_id = window_id
    """
    window_id child of solution_animations_child.
    """
    view: view = view
    """
    view child of solution_animations_child.
    """
    use_raytracing: use_raytracing = use_raytracing
    """
    use_raytracing child of solution_animations_child.
    """
    command_names = \
        ['display']

    display: display = display
    """
    display command of solution_animations_child.
    """



"""Wrappers over SchemeEval gRPC service of Fluent.

Example
-------
>>> from ansys.fluent.core.services.scheme_eval import Symbol as S
>>> session.scheme_eval.eval([S('+'), 2, 3])
5
>>> session.scheme_eval.eval([S('rpgetvar'), [S('string->symbol'), "mom/relax"]])  # noqa: E501
0.7
>>> session.scheme_eval.exec(('(ti-menu-load-string "/report/system/proc-stats")',))  # noqa: E501
>>> # Returns TUI output string
>>> session.scheme_eval.string_eval("(+ 2 3)")
'5'
>>> session.scheme_eval.string_eval("(rpgetvar 'mom/relax)")
'0.7'
>>> session.scheme_eval.scheme_eval("(+ 2 3)")
5
>>> session.scheme_eval.scheme_eval("(rpgetvar 'mom/relax)")
0.7
"""

from typing import Any, List, Sequence, Tuple

import grpc

from ansys.api.fluent.v0 import scheme_eval_pb2 as SchemeEvalProtoModule
from ansys.api.fluent.v0 import scheme_eval_pb2_grpc as SchemeEvalGrpcModule
from ansys.api.fluent.v0.scheme_pointer_pb2 import SchemePointer


class SchemeEvalService:
    """Class wrapping the SchemeEval gRPC service of Fluent.

    Using the methods from the SchemeEval class is recommended.
    """

    def __init__(self, channel: grpc.Channel, metadata: List[Tuple[str, str]]):
        self.__stub = SchemeEvalGrpcModule.SchemeEvalStub(channel)
        self.__metadata = metadata

    def eval(self, request: SchemePointer) -> SchemePointer:
        return self.__stub.Eval(request, metadata=self.__metadata)

    def exec(
        self, request: SchemeEvalProtoModule.ExecRequest
    ) -> SchemeEvalProtoModule.ExecResponse:
        return self.__stub.Exec(request, metadata=self.__metadata)

    def string_eval(
        self, request: SchemeEvalProtoModule.StringEvalRequest
    ) -> SchemeEvalProtoModule.StringEvalResponse:
        return self.__stub.StringEval(request, metadata=self.__metadata)

    def scheme_eval(
        self, request: SchemeEvalProtoModule.SchemeEvalRequest
    ) -> SchemeEvalProtoModule.SchemeEvalResponse:
        return self.__stub.SchemeEval(request, metadata=self.__metadata)


class Symbol:
    """Class representing the symbol datatype in Fluent.

    Attributes
    ----------
    str : str
        Underlying string representation
    """

    def __init__(self, str: str):
        self.str = str


def _convert_pair_to_scheme_pointer(val: Tuple[Any, Any], p: SchemePointer, version):
    _convert_py_value_to_scheme_pointer(val[0], p.pair.car, version)
    _convert_py_value_to_scheme_pointer(val[1], p.pair.cdr, version)


def _convert_list_of_pairs_to_scheme_pointer(
    val: List[Tuple[Any, Any]], p: SchemePointer, version
):
    if len(val) > 0:
        _convert_pair_to_scheme_pointer(val[0], p.pair.car, version)
        _convert_list_of_pairs_to_scheme_pointer(val[1:], p.pair.cdr, version)


def _convert_py_value_to_scheme_pointer(val: Any, p: SchemePointer, version):
    """Convert Python datatype to Scheme pointer."""
    if isinstance(val, bool):
        p.b = val
    elif isinstance(val, int):
        p.fixednum = val
    elif isinstance(val, float):
        p.flonum = val
    elif isinstance(val, str):
        p.str = val
    elif isinstance(val, Symbol):
        p.sym = val.str
    elif isinstance(val, tuple) and len(val) == 2:
        _convert_py_value_to_scheme_pointer(val[0], p.pair.car, version)
        _convert_py_value_to_scheme_pointer(val[1], p.pair.cdr, version)
    elif isinstance(val, list) or isinstance(val, tuple):
        if version < "23.1":
            if val:
                val = list(val)
                _convert_py_value_to_scheme_pointer(val[0], p.pair.car, version)
                _convert_py_value_to_scheme_pointer(val[1:], p.pair.cdr, version)
        else:
            for item in val:
                _convert_py_value_to_scheme_pointer(item, p.list.item.add(), version)
    elif isinstance(val, dict):
        if version < "23.1":
            as_list = list(val.items())
            if as_list:
                _convert_pair_to_scheme_pointer(as_list[0], p.pair.car, version)
                _convert_list_of_pairs_to_scheme_pointer(
                    as_list[1:], p.pair.cdr, version
                )
        else:
            for k, v in val.items():
                item = p.list.item.add()
                _convert_py_value_to_scheme_pointer(k, item.pair.car, version)
                _convert_py_value_to_scheme_pointer(v, item.pair.cdr, version)


def _convert_scheme_pointer_to_py_list(p: SchemePointer, version):
    val = []
    val.append(_convert_scheme_pointer_to_py_value(p.pair.car, version))
    if p.pair.cdr.HasField("pair"):
        tail = _convert_scheme_pointer_to_py_list(p.pair.cdr, version)
        val.extend([tail] if isinstance(tail, dict) else tail)
    if all(
        isinstance(x, dict)
        or (
            (isinstance(x, tuple) or isinstance(x, list))
            and x
            and isinstance(x[0], str)
        )
        for x in val
    ):
        d = {}
        for x in val:
            if isinstance(x, dict):
                d.update(x)
            else:
                d[x[0]] = x[1:] if len(x) > 2 else x[1]
        return d
    return val


def _convert_scheme_pointer_to_py_value(p: SchemePointer, version):
    """Convert Scheme pointer to Python datatype."""
    if p.HasField("b"):
        return p.b
    elif p.HasField("fixednum"):
        return p.fixednum
    elif p.HasField("flonum"):
        return p.flonum
    elif p.HasField("c"):
        return p.c
    elif p.HasField("str"):
        return p.str
    elif p.HasField("sym"):
        return Symbol(p.sym)
    elif p.HasField("pair"):
        if version < "23.1":
            if any(
                p.pair.cdr.HasField(x)
                for x in ["b", "fixednum", "flonum", "c", "str", "sym"]
            ):
                return (
                    _convert_scheme_pointer_to_py_value(p.pair.car, version),
                    _convert_scheme_pointer_to_py_value(p.pair.cdr, version),
                )
            else:
                val = _convert_scheme_pointer_to_py_list(p, version)
                return val
        else:
            car = _convert_scheme_pointer_to_py_value(p.pair.car, version)
            cdr = _convert_scheme_pointer_to_py_value(p.pair.cdr, version)
            return (car,) if cdr is None else (car, cdr)
    elif p.HasField("list"):
        is_dict = all(item.HasField("pair") for item in p.list.item)
        if is_dict:
            return {
                _convert_scheme_pointer_to_py_value(
                    item.pair.car, version
                ): _convert_scheme_pointer_to_py_value(item.pair.cdr, version)
                for item in p.list.item
            }
        else:
            return [
                _convert_scheme_pointer_to_py_value(item, version)
                for item in p.list.item
            ]

    return None


class SchemeEval:
    """Class on which Fluent's scheme code can be executed.

    Methods
    -------
    eval(val)
        Evaluates a scheme expression, returns Python value
    exec(commands, wait, silent)
        Executes a sequence of scheme commands, returns TUI output
        string
    string_eval(input)
        Evaluates a scheme expression in string format, returns string
    scheme_eval(input)
        Evaluates a scheme expression in string format, returns Python
        value
    """

    def __init__(self, service: SchemeEvalService, version=None):
        self.service = service
        self.version = version

    def eval(self, val: Any) -> Any:
        """Evaluates a scheme expression.

        Parameters
        ----------
        val : Any
            Input scheme expression represented as Python datatype

        Returns
        -------
        Any
            Output scheme value represented as Python datatype
        """
        if self.version < "23.1":
            request = SchemePointer()
            _convert_py_value_to_scheme_pointer(val, request, self.version)
            response = self.service.eval(request)
            return _convert_scheme_pointer_to_py_value(response, self.version)
        else:
            request = SchemeEvalProtoModule.SchemeEvalRequest()
            _convert_py_value_to_scheme_pointer(val, request.input, self.version)
            response = self.service.scheme_eval(request)
            return _convert_scheme_pointer_to_py_value(response.output, self.version)

    def exec(
        self, commands: Sequence[str], wait: bool = True, silent: bool = True
    ) -> str:
        """Executes a sequence of scheme commands.

        Parameters
        ----------
        commands : Sequence[str]
            Sequence of scheme commands in string format
        wait : bool, optional
            Specifies whether to wait until execution completes, by
            default True
        silent : bool, optional
            Specifies whether to execute silently, by default True

        Returns
        -------
        str
           Output as string
        """
        request = SchemeEvalProtoModule.ExecRequest()
        request.command.extend(commands)
        request.wait = wait
        request.silent = silent
        response = self.service.exec(request)
        return response.output

    def string_eval(self, input: str) -> str:
        """Evaluates a scheme expression in string format.

        Parameters
        ----------
        input : str
            Input scheme expression in string format

        Returns
        -------
        str
            Output scheme value in string format
        """
        request = SchemeEvalProtoModule.StringEvalRequest()
        request.input = input
        response = self.service.string_eval(request)
        return response.output

    def scheme_eval(self, input: str) -> Any:
        """Evaluates a scheme expression in string format.

        Parameters
        ----------
        input : str
            Input scheme expression in string format

        Returns
        -------
        str
            Output scheme value represented as Python datatype
        """
        S = Symbol  # noqa N806
        val = (
            S("eval"),
            (S("with-input-from-string"), input, S("read")),
            S("user-initial-environment"),
        )
        return self.eval(val)



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .option_9 import option
from .value import value
from .piecewise_linear import piecewise_linear
from .piecewise_polynomial import piecewise_polynomial
from .polynomial import polynomial
from .expression import expression
from .power_law import power_law
from .sutherland import sutherland
from .user_defined_function import user_defined_function
from .real_gas_nist_mixture import real_gas_nist_mixture
class viscosity(Group):
    """
    'viscosity' child.
    """

    fluent_name = "viscosity"

    child_names = \
        ['option', 'value', 'piecewise_linear', 'piecewise_polynomial',
         'polynomial', 'expression', 'power_law', 'sutherland',
         'user_defined_function', 'real_gas_nist_mixture']

    option: option = option
    """
    option child of viscosity.
    """
    value: value = value
    """
    value child of viscosity.
    """
    piecewise_linear: piecewise_linear = piecewise_linear
    """
    piecewise_linear child of viscosity.
    """
    piecewise_polynomial: piecewise_polynomial = piecewise_polynomial
    """
    piecewise_polynomial child of viscosity.
    """
    polynomial: polynomial = polynomial
    """
    polynomial child of viscosity.
    """
    expression: expression = expression
    """
    expression child of viscosity.
    """
    power_law: power_law = power_law
    """
    power_law child of viscosity.
    """
    sutherland: sutherland = sutherland
    """
    sutherland child of viscosity.
    """
    user_defined_function: user_defined_function = user_defined_function
    """
    user_defined_function child of viscosity.
    """
    real_gas_nist_mixture: real_gas_nist_mixture = real_gas_nist_mixture
    """
    real_gas_nist_mixture child of viscosity.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .gravity import gravity
from .components import components
class gravity(Group):
    """
    'gravity' child.
    """

    fluent_name = "gravity"

    child_names = \
        ['gravity', 'components']

    gravity: gravity = gravity
    """
    gravity child of gravity.
    """
    components: components = components
    """
    components child of gravity.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .mesh_child import mesh_child

class mesh(NamedObject[mesh_child], _CreatableNamedObjectMixin[mesh_child]):
    """
    'mesh' child.
    """

    fluent_name = "mesh"

    child_object_type: mesh_child = mesh_child
    """
    child_object_type of mesh.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class include_lwf_particles_in_dpm_concentration(Boolean):
    """
    Include/exclude LWF particle mass in the calculation of DPM Concentration.
    """

    fluent_name = "include-lwf-particles-in-dpm-concentration?"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .option_3 import option
from .max_number_of_refinements import max_number_of_refinements
from .tolerance import tolerance
class accuracy_control(Group):
    """
    'accuracy_control' child.
    """

    fluent_name = "accuracy-control"

    child_names = \
        ['option', 'max_number_of_refinements', 'tolerance']

    option: option = option
    """
    option child of accuracy_control.
    """
    max_number_of_refinements: max_number_of_refinements = max_number_of_refinements
    """
    max_number_of_refinements child of accuracy_control.
    """
    tolerance: tolerance = tolerance
    """
    tolerance child of accuracy_control.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class sbes_sdes_hybrid_model_optn(String):
    """
    Select the SBES / SDES hybrid model.
    """

    fluent_name = "sbes-sdes-hybrid-model-optn"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .network_child import network_child

class network(NamedObject[network_child], _NonCreatableNamedObjectMixin[network_child]):
    """
    'network' child.
    """

    fluent_name = "network"

    child_object_type: network_child = network_child
    """
    child_object_type of network.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .option_2 import option
from .b import b
from .reference_viscosity import reference_viscosity
from .reference_temperature import reference_temperature
from .temperature_exponent import temperature_exponent
class power_law(Group):
    """
    'power_law' child.
    """

    fluent_name = "power-law"

    child_names = \
        ['option', 'b', 'reference_viscosity', 'reference_temperature',
         'temperature_exponent']

    option: option = option
    """
    option child of power_law.
    """
    b: b = b
    """
    b child of power_law.
    """
    reference_viscosity: reference_viscosity = reference_viscosity
    """
    reference_viscosity child of power_law.
    """
    reference_temperature: reference_temperature = reference_temperature
    """
    reference_temperature child of power_law.
    """
    temperature_exponent: temperature_exponent = temperature_exponent
    """
    temperature_exponent child of power_law.
    """



"""Setup file for ansys-fluent-core."""
import os
import shutil

from setuptools import find_namespace_packages, setup

# Get version from version info
__version__ = None
_THIS_FILE = os.path.dirname(__file__)
_VERSION_FILE = os.path.join(
    _THIS_FILE, "src", "ansys", "fluent", "core", "_version.py"
)
with open(_VERSION_FILE, mode="r", encoding="utf8") as fd:
    # execute file from raw string
    exec(fd.read())

# Copy README.rst file to docs folder in ansys.fluent.core
_README_FILE = os.path.join(_THIS_FILE, "README.rst")
_DOCS_FILE = os.path.join(
    _THIS_FILE, "src", "ansys", "fluent", "core", "docs", "README.rst"
)
shutil.copy2(_README_FILE, _DOCS_FILE)

install_requires = [
    "ansys-api-fluent==0.3.5",
    "ansys-platform-instancemanagement~=1.0",
    "grpcio>=1.30.0",
    "numpy>=1.21.5",
    "appdirs>=1.4.0",
    "pandas>=1.1.5",
    "h5py>=3.7.0",
]


packages = []
for package in find_namespace_packages(where="src", include="ansys*"):
    if package.startswith("ansys.fluent"):
        packages.append(package)

setup(
    name="ansys-fluent-core",
    packages=packages,
    package_dir={"": "src"},
    include_package_data=True,
    version=__version__,
    description="Pythonic interface to Ansys Fluent",
    long_description=open(_README_FILE, encoding="utf8").read(),
    long_description_content_type="text/x-rst",
    license="MIT",
    author="ANSYS, Inc.",
    author_email="pyansys.support@ansys.com",
    maintainer="PyAnsys developers",
    maintainer_email="pyansys.maintainers@ansys.com",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    url="https://github.com/pyansys/pyfluent",
    python_requires=">=3.7",
    install_requires=install_requires,
)



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class disable_warped_face_gradient_correction(Command):
    """
    'disable_warped_face_gradient_correction' command.
    """

    fluent_name = "disable-warped-face-gradient-correction"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class rke_cmu_rotation_term(Boolean):
    """
    Enable/disable inclusion of omega in the Cmu definition.
    """

    fluent_name = "rke-cmu-rotation-term?"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .use_weighting import use_weighting
from .make_steady_from_unsteady_file import make_steady_from_unsteady_file
from .weighting_variable import weighting_variable
from .reset_min_and_max import reset_min_and_max
from .set_minimum import set_minimum
from .set_maximum import set_maximum
from .use_logarithmic import use_logarithmic
from .number_of_bins import number_of_bins
from .all_variables_number_of_bins import all_variables_number_of_bins
from .list_settings import list_settings
class setup_reduction(Group):
    """
    Set up the sample data reduction by specifying all relevant options and setting parameters as desired.
    """

    fluent_name = "setup-reduction"

    child_names = \
        ['use_weighting', 'make_steady_from_unsteady_file']

    use_weighting: use_weighting = use_weighting
    """
    use_weighting child of setup_reduction.
    """
    make_steady_from_unsteady_file: make_steady_from_unsteady_file = make_steady_from_unsteady_file
    """
    make_steady_from_unsteady_file child of setup_reduction.
    """
    command_names = \
        ['weighting_variable', 'reset_min_and_max', 'set_minimum',
         'set_maximum', 'use_logarithmic', 'number_of_bins',
         'all_variables_number_of_bins', 'list_settings']

    weighting_variable: weighting_variable = weighting_variable
    """
    weighting_variable command of setup_reduction.
    """
    reset_min_and_max: reset_min_and_max = reset_min_and_max
    """
    reset_min_and_max command of setup_reduction.
    """
    set_minimum: set_minimum = set_minimum
    """
    set_minimum command of setup_reduction.
    """
    set_maximum: set_maximum = set_maximum
    """
    set_maximum command of setup_reduction.
    """
    use_logarithmic: use_logarithmic = use_logarithmic
    """
    use_logarithmic command of setup_reduction.
    """
    number_of_bins: number_of_bins = number_of_bins
    """
    number_of_bins command of setup_reduction.
    """
    all_variables_number_of_bins: all_variables_number_of_bins = all_variables_number_of_bins
    """
    all_variables_number_of_bins command of setup_reduction.
    """
    list_settings: list_settings = list_settings
    """
    list_settings command of setup_reduction.
    """



"""Version of ansys-fluent-core module.

On the ``main`` branch, use 'dev0' to denote a development version.
For example:
version_info = 0, 1, 'dev0'
"""

# major, minor, patch
version_info = 0, 12, "dev6"

# Nice string for the version
__version__ = ".".join(map(str, version_info))



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .execute_settings_optimization import execute_settings_optimization
from .execute_advanced_stabilization import execute_advanced_stabilization
from .additional_stabilization_controls import additional_stabilization_controls
from .execute_additional_stability_controls import execute_additional_stability_controls
from .velocity_limiting_treatment import velocity_limiting_treatment
class solution_stabilization(Group):
    """
    VOF solution stabilization menu.
    """

    fluent_name = "solution-stabilization"

    child_names = \
        ['execute_settings_optimization', 'execute_advanced_stabilization',
         'additional_stabilization_controls',
         'execute_additional_stability_controls',
         'velocity_limiting_treatment']

    execute_settings_optimization: execute_settings_optimization = execute_settings_optimization
    """
    execute_settings_optimization child of solution_stabilization.
    """
    execute_advanced_stabilization: execute_advanced_stabilization = execute_advanced_stabilization
    """
    execute_advanced_stabilization child of solution_stabilization.
    """
    additional_stabilization_controls: additional_stabilization_controls = additional_stabilization_controls
    """
    additional_stabilization_controls child of solution_stabilization.
    """
    execute_additional_stability_controls: execute_additional_stability_controls = execute_additional_stability_controls
    """
    execute_additional_stability_controls child of solution_stabilization.
    """
    velocity_limiting_treatment: velocity_limiting_treatment = velocity_limiting_treatment
    """
    velocity_limiting_treatment child of solution_stabilization.
    """



"""Module for accessing and modifying hierarchy of Fluent settings.

The only useful method is '`get_root``, which returns the root object for
accessing Fluent settings.

Child objects can be generally accessed or modified using attribute access.
Named child objects can be accessed or modified using index operators.

Calling an object will return its current value.

Example
-------
r = flobject.get_root(proxy)
is_energy_on = r.setup.models.energy.enabled()
r.setup.models.energy.enabled = True
r.boundary_conditions.velocity_inlet['inlet'].vmag.constant = 20
"""
import collections
import hashlib
import importlib
import keyword
import pickle
import string
import sys
from typing import Any, Dict, Generic, List, NewType, Tuple, TypeVar, Union
import weakref

from .logging import LOG

# Type hints
RealType = NewType("real", Union[float, str])  # constant or expression
RealListType = List[RealType]
RealVectorType = Tuple[RealType, RealType, RealType]
IntListType = List[int]
StringListType = List[str]
BoolListType = List[bool]
PrimitiveStateType = Union[
    str,
    RealType,
    int,
    bool,
    RealListType,
    IntListType,
    StringListType,
    BoolListType,
]
DictStateType = Dict[str, "StateType"]
ListStateType = List["StateType"]
StateType = Union[PrimitiveStateType, DictStateType, ListStateType]

_ttable = str.maketrans(string.punctuation, "_" * len(string.punctuation), "?'")


def to_python_name(fluent_name: str) -> str:
    """Convert a scheme string to a Python variable name.

    This function replaces symbols with _. Any ``?`` symbols are
    ignored.
    """
    if not fluent_name:
        return fluent_name
    name = fluent_name.translate(_ttable)
    while name in keyword.kwlist:
        name = name + "_"
    return name


class Base:
    """Base class for settings and command objects.

    Parameters
    ----------
    name : str
        Name of the object if a child of a named object.
    parent: Base
           Parent of the object.

    Attributes
    ----------
    flproxy
    obj_name
    fluent_name
    """

    def __init__(self, name: str = None, parent=None):
        """__init__ of Base class."""
        self._setattr("_parent", weakref.proxy(parent) if parent is not None else None)
        self._setattr("_flproxy", None)
        if name is not None:
            self._setattr("_name", name)

    def set_flproxy(self, flproxy):
        """Set flproxy object."""
        self._setattr("_flproxy", flproxy)

    @property
    def flproxy(self):
        """Proxy object.

        The proxy object is set at the root level and accessed via the
        parent for the child classes.
        """
        if self._flproxy is None:
            return self._parent.flproxy
        return self._flproxy

    _name = None
    fluent_name = None

    @property
    def parent(self):
        """Parent (container) object."""
        return self._parent

    @property
    def obj_name(self) -> str:
        """Name of the scheme of this object.

        By default, this returns the object's static name. If the object
        is a child of a named object, the object's name is returned.
        """
        if self._name is None:
            return self.fluent_name
        return self._name

    @property
    def path(self) -> str:
        """Path of the object.

        Constructed from the ``obj_name`` of self and the path of
        parent.
        """
        if self._parent is None:
            return self.obj_name
        ppath = self._parent.path
        if not ppath:
            return self.obj_name
        return ppath + "/" + self.obj_name

    def get_attrs(self, attrs) -> Any:
        """Get the requested attributes for the object."""
        return self.flproxy.get_attrs(self.path, attrs)

    def get_attr(self, attr, attr_type_or_types=None) -> Any:
        """Get the requested attribute for the object."""
        attrs = self.get_attrs([attr])
        if attrs:
            attrs = attrs.get("attrs", attrs)
        if attr != "active?" and attrs and attrs.get("active?", True) is False:
            raise RuntimeError("Object is not active")
        val = None
        if attrs:
            val = attrs[attr]

        if attr_type_or_types:
            if not type(attr_type_or_types) == tuple:
                attr_type_or_types = (attr_type_or_types,)
            if type(val) in attr_type_or_types:
                return val
            return None
        return val

    def is_active(self) -> bool:
        """Whether the object is active."""
        return self.get_attr("active?", bool)

    def is_read_only(self) -> bool:
        """Whether the object is read-only."""
        return self.get_attr("read-only?", bool)

    def __setattr__(self, name, value):
        raise AttributeError(name)

    # __setattr__ is overridden to prevent creation of new attributes or
    # overriding existing ones. _setattr is the backdoor to set attributes
    def _setattr(self, name, value):
        super().__setattr__(name, value)


StateT = TypeVar("StateT")


class Property(Base):
    """Exposes attribute accessor on settings object."""

    def default_value(self):
        """Gets the default value of the object."""
        return self.get_attr("default")


class Numerical(Property):
    """Exposes attribute accessor on settings object - specific to numerical objects."""

    def min(self):
        """Get the minimum value of the object."""
        val = self.get_attr("min", (float, int))
        return None if isinstance(val, bool) else val

    def max(self):
        """Get the maximum value of the object."""
        val = self.get_attr("max", (float, int))
        return None if isinstance(val, bool) else val


class Textual(Property):
    """Exposes attribute accessor on settings object - specific to string objects."""

    def allowed_values(self):
        """Get the allowed values of the object."""
        return self.get_attr("allowed-values", (list, str))


class SettingsBase(Base, Generic[StateT]):
    """Base class for settings objects.

    Methods
    -------
    get_state()
        Get the current state of the object.

    set_state(state)
        Set the state of the object.
    """

    @classmethod
    def to_scheme_keys(cls, value: StateT) -> StateT:
        """Convert value to have keys with scheme names.

        This is overridden in the ``Group``, ``NamedObject``, and
        ``ListObject`` classes.
        """
        return value

    @classmethod
    def to_python_keys(cls, value: StateT) -> StateT:
        """Convert value to have keys with Python names.

        This is overridden in the ``Group``, ``NamedObject``, and
        ``ListObject`` classes.
        """
        return value

    def __call__(self) -> StateT:
        """Alias for self.get_state."""
        return self.get_state()

    def get_state(self) -> StateT:
        """Get the state of the object."""
        return self.to_python_keys(self.flproxy.get_var(self.path))

    def set_state(self, state: StateT = None, **kwargs):
        """Set the state of the object."""
        if kwargs:
            return self.flproxy.set_var(self.path, self.to_scheme_keys(kwargs))
        else:
            return self.flproxy.set_var(self.path, self.to_scheme_keys(state))

    @staticmethod
    def _print_state_helper(state, out, indent=0, indent_factor=2):
        if isinstance(state, dict):
            out.write("\n")
            for key, value in state.items():
                if value is not None:
                    out.write(f'{indent*indent_factor*" "}{key} : ')
                    SettingsBase._print_state_helper(
                        value, out, indent + 1, indent_factor
                    )
        elif isinstance(state, list):
            out.write("\n")
            for index, value in enumerate(state):
                out.write(f'{indent*indent_factor*" "}{index} : ')
                SettingsBase._print_state_helper(value, out, indent + 1, indent_factor)
        else:
            out.write(f"{state}\n")

    def print_state(self, out=None, indent_factor=2):
        """Print the state of the object."""
        out = sys.stdout if out is None else out
        self._print_state_helper(self.get_state(), out, indent_factor=indent_factor)


class Integer(SettingsBase[int], Numerical):
    """An ``Integer`` object representing an integer value setting."""

    _state_type = int


class Real(SettingsBase[RealType], Numerical):
    """A ``Real`` object representing a real value setting.

    Some ``Real`` objects also accept string arguments representing
    expression values.
    """

    _state_type = RealType


class String(SettingsBase[str], Textual):
    """A ``String`` object representing a string value setting."""

    _state_type = str


class Filename(SettingsBase[str], Textual):
    """A ``Filename`` object representing a file name."""

    _state_type = str


class FilenameList(SettingsBase[StringListType], Textual):
    """A FilenameList object represents a list of file names."""

    _state_type = StringListType


class Boolean(SettingsBase[bool], Property):
    """A ``Boolean`` object representing a Boolean value setting."""

    _state_type = bool


class RealList(SettingsBase[RealListType], Numerical):
    """A ``RealList`` object representing a real list setting."""

    _state_type = RealListType


class IntegerList(SettingsBase[IntListType], Numerical):
    """An ``Integer`` object representing an integer list setting."""

    _state_type = IntListType


class RealVector(SettingsBase[RealVectorType], Numerical):
    """An object representing a 3D vector.

    A ``RealVector`` object representing a real vector setting
    consisting of three real values.
    """

    _state_type = RealVectorType


class StringList(SettingsBase[StringListType], Textual):
    """A ``StringList`` object representing a string list setting."""

    _state_type = StringListType


class BooleanList(SettingsBase[BoolListType], Property):
    """A ``BooleanList`` object representing a Boolean list setting."""

    _state_type = BoolListType


class Group(SettingsBase[DictStateType]):
    """A ``Group`` container object.

    A ``Group`` object is a container similar to a C++ structure object.
    Child objects can be accessed via attribute access.

    Attributes
    ----------
    child_names: list[str]
        Names of the child objects
    command_names: list[str]
        Names of the commands
    """

    _state_type = DictStateType

    def __init__(self, name: str = None, parent=None):
        """__init__ of Group class."""
        super().__init__(name, parent)
        for child in self.child_names:
            cls = getattr(self.__class__, child)
            self._setattr(child, cls(None, self))
        for cmd in self.command_names:
            cls = getattr(self.__class__, cmd)
            self._setattr(cmd, cls(None, self))
        for query in self.query_names:
            cls = getattr(self.__class__, query)
            self._setattr(query, cls(None, self))

    def __call__(self, *args, **kwargs):
        if kwargs:
            self.set_state(kwargs)
        elif args:
            self.set_state(args)
        else:
            return self.get_state()

    @classmethod
    def to_scheme_keys(cls, value):
        """Convert value to have keys with scheme names."""
        if isinstance(value, collections.abc.Mapping):
            ret = {}
            for k, v in value.items():
                if k in cls.child_names:
                    ccls = getattr(cls, k)
                    ret[ccls.fluent_name] = ccls.to_scheme_keys(v)
                else:
                    raise RuntimeError("Key '" + str(k) + "' is invalid")
            return ret
        else:
            return value

    @classmethod
    def to_python_keys(cls, value):
        """Convert value to have keys with Python names."""
        if isinstance(value, collections.abc.Mapping):
            ret = {}
            undef = object()
            for mname in cls.child_names:
                ccls = getattr(cls, mname)
                mvalue = value.get(ccls.fluent_name, undef)
                if mvalue is not undef:
                    ret[mname] = ccls.to_python_keys(mvalue)
            return ret
        else:
            return value

    child_names = []
    command_names = []
    query_names = []

    def get_active_child_names(self):
        """Names of children that are currently active."""
        ret = []
        for child in self.child_names:
            if getattr(self, child).is_active():
                ret.append(child)
        return ret

    def get_active_command_names(self):
        """Names of commands that are currently active."""
        ret = []
        for command in self.command_names:
            if getattr(self, command).is_active():
                ret.append(command)
        return ret

    def get_active_query_names(self):
        """Names of queries that are currently active."""
        ret = []
        for query in self.query_names:
            if getattr(self, query).is_active():
                ret.append(query)
        return ret

    def __getattribute__(self, name):
        if name in super().__getattribute__("child_names"):
            if not self.is_active():
                raise RuntimeError(f"'{self.path}' is currently not active")
        return super().__getattribute__(name)

    def __setattr__(self, name: str, value):
        return getattr(self, name).set_state(value)


ChildTypeT = TypeVar("ChildTypeT")


class NamedObject(SettingsBase[DictStateType], Generic[ChildTypeT]):
    """A ``NamedObject`` container is a container object similar to a Python
    dictionary object. Generally, many such objects can be created with
    different names.

    Attributes
    ----------
    command_names: list[str]
        Names of the commands
    """

    # New objects could get inserted by other operations, so we cannot assume
    # that the local cache in self._objects is always up-to-date
    def __init__(self, name: str = None, parent=None):
        """__init__ of NamedObject class."""
        super().__init__(name, parent)
        self._setattr("_objects", {})
        for cmd in self.command_names:
            cls = getattr(self.__class__, cmd)
            self._setattr(cmd, cls(None, self))
        for query in self.query_names:
            cls = getattr(self.__class__, query)
            self._setattr(query, cls(None, self))

    @classmethod
    def to_scheme_keys(cls, value):
        """Convert value to have keys with scheme names."""
        if isinstance(value, collections.abc.Mapping):
            ret = {}
            for k, v in value.items():
                ret[k] = cls.child_object_type.to_scheme_keys(v)
            return ret
        else:
            return value

    @classmethod
    def to_python_keys(cls, value):
        """Convert value to have keys with Python names."""
        if isinstance(value, collections.abc.Mapping):
            ret = {}
            for k, v in value.items():
                ret[k] = cls.child_object_type.to_python_keys(v)
            return ret
        else:
            return value

    command_names = []
    query_names = []

    def _create_child_object(self, cname: str):
        ret = self._objects.get(cname)
        if not ret:
            cls = self.__class__.child_object_type
            ret = self._objects[cname] = cls(cname, self)
        return ret

    def _update_objects(self):
        names = self.get_object_names()
        for name in list(self._objects.keys()):
            if name not in names:
                del self._objects[name]
        for name in names:
            if name not in self._objects:
                self._create_child_object(name)

    def rename(self, new: str, old: str):
        """Rename a named object.

        Parameters
        ----------
        new: str
            New name.
        old : str
            Current name.
        """
        self.flproxy.rename(self.path, new, old)
        if old in self._objects:
            del self._objects[old]
        self._create_child_object(new)

    def __delitem__(self, name: str):
        self.flproxy.delete(self.path, name)
        if name in self._objects:
            del self._objects[name]

    def __contains__(self, name: str):
        return name in self.get_object_names()

    def __len__(self):
        return len(self.keys())

    def __iter__(self):
        self._update_objects()
        return iter(self._objects)

    def keys(self):
        """Object names."""
        self._update_objects()
        return self._objects.keys()

    def values(self):
        """Object values."""
        self._update_objects()
        return self._objects.values()

    def items(self):
        """Items."""
        self._update_objects()
        return self._objects.items()

    def user_creatable(self) -> bool:
        """Whether the object is user-creatable."""
        return self.get_attr("user-creatable?", bool)

    def get_object_names(self):
        """Object names."""
        obj_names = self.flproxy.get_object_names(self.path)
        obj_names_list = obj_names if isinstance(obj_names, list) else list(obj_names)
        return obj_names_list

    def __getitem__(self, name: str) -> ChildTypeT:
        if name not in self.get_object_names():
            raise KeyError(name)
        obj = self._objects.get(name)
        if not obj:
            obj = self._create_child_object(name)
        return obj


class ListObject(SettingsBase[ListStateType], Generic[ChildTypeT]):
    """A ``ListObject`` container is a container object, similar to a Python
    list object. Generally, many such objects can be created.

    Attributes
    ----------
    command_names: list[str]
        Names of the commands.

    Methods
    -------
    get_size()
       Get the size of the list.

    resize(size)
        Resize the list.
    """

    # New objects could get inserted by other operations, so we cannot assume
    # that the local cache in self._objects is always up-to-date
    def __init__(self, name=None, parent=None):
        """__init__ of ListObject class."""
        super().__init__(name, parent)
        self._setattr("_objects", [])
        for cmd in self.command_names:
            cls = getattr(self.__class__, cmd)
            self._setattr(cmd, cls(None, self))
        for query in self.query_names:
            cls = getattr(self.__class__, query)
            self._setattr(query, cls(None, self))

    @classmethod
    def to_scheme_keys(cls, value):
        """Convert value to have keys with scheme names."""
        if isinstance(value, collections.abc.Sequence):
            return [cls.child_object_type.to_scheme_keys(v) for v in value]
        else:
            return value

    @classmethod
    def to_python_keys(cls, value):
        """Convert value to have keys with scheme names."""
        if isinstance(value, collections.abc.Sequence):
            return [cls.child_object_type.to_python_keys(v) for v in value]
        else:
            return value

    command_names = []
    query_names = []

    def _update_objects(self):
        cls = self.__class__.child_object_type
        self._setattr("_objects", [cls(str(x), self) for x in range(self.get_size())])

    def __len__(self):
        return self.get_size()

    def __iter__(self):
        self._update_objects()
        return iter(self._objects)

    def get_size(self) -> int:
        """Return the number of elements in a list object.

        Returns
        -------
        int
        """
        return self.flproxy.get_list_size(self.path)

    def resize(self, size: int):
        """Resize the list object.

        Parameters
        ----------
        size: int
            New size
        """
        self.flproxy.resize_list_object(self.path, size)

    def __getitem__(self, index: int) -> ChildTypeT:
        size = self.get_size()
        if index >= size:
            raise IndexError(index)
        if len(self._objects) != size:
            self._update_objects()
        return self._objects[index]

    def __setitem__(self, index: int, value):
        child = self[index]
        child.set_state(value)


class Map(SettingsBase[DictStateType]):
    """A ``Map`` object representing key-value settings."""


def _get_new_keywords(obj, kwds):
    newkwds = {}
    for k, v in kwds.items():
        if k in obj.argument_names:
            ccls = getattr(obj, k)
            newkwds[ccls.fluent_name] = ccls.to_scheme_keys(v)
        else:
            raise RuntimeError("Argument '" + str(k) + "' is invalid")
    return newkwds


class Action(Base):
    """Intermediate Base class for Command and Query class."""

    def __init__(self, name: str = None, parent=None):
        """__init__ of Action class."""
        super().__init__(name, parent)
        if hasattr(self, "argument_names"):
            for argument in self.argument_names:
                cls = getattr(self.__class__, argument)
                self._setattr(argument, cls(None, self))

    def arguments(self) -> Any:
        """Get the arguments for the Action."""
        attrs = self.get_attrs(["arguments"])
        if attrs:
            attrs = attrs.get("attrs", attrs)
        if attrs and attrs.get("active?", True) is False:
            raise RuntimeError(f"{self.__class__.__name__} is not active")
        return attrs["arguments"] if attrs else None


class Command(Action):
    """Command object."""

    def __call__(self, **kwds):
        """Call a query with the specified keyword arguments."""
        newkwds = _get_new_keywords(self, kwds)
        return self.flproxy.execute_cmd(self._parent.path, self.obj_name, **newkwds)


class Query(Action):
    """Query object."""

    def __call__(self, **kwds):
        """Call a query with the specified keyword arguments."""
        newkwds = _get_new_keywords(self, kwds)
        return self.flproxy.execute_query(self._parent.path, self.obj_name, **newkwds)


_baseTypes = {
    "group": Group,
    "integer": Integer,
    "real": Real,
    "string/symbol": String,
    "string": String,
    "boolean": Boolean,
    "real-list": RealList,
    "integer-list": IntegerList,
    "string-list": StringList,
    "boolean-list": BooleanList,
    "named-object": NamedObject,
    "vector": RealVector,
    "command": Command,
    "query": Query,
    "material-property": String,
    "thread-var": String,
    "list-object": ListObject,
    "file": Filename,
    "file-list": FilenameList,
    "map": Map,
}


def _clean_helpinfo(helpinfo):
    helpinfo = helpinfo.strip("\n")
    if not helpinfo.endswith("."):
        helpinfo += "."
    helpinfo = helpinfo[0].upper() + helpinfo[1:]
    return helpinfo


class _ChildNamedObjectAccessorMixin(collections.abc.MutableMapping):
    """A mixin class to provide a dictionary interface at a Group class level
    if the Group has multiple named objects of a similar type. For example,
    boundary conditions are grouped by type but quite often we want to access
    them without the type context.

    The following can be used:
    for name, boundary in setup.boundary_conditions.items():
        print (name, boundary())

    even though actual boundary conditions are stored one level lower to
    boundary_conditions.
    """

    def __getitem__(self, name):
        """Get a child object."""
        for cname in self.child_names:
            cobj = getattr(self, cname)
            try:
                return cobj[name]
            except Exception:
                pass
        raise KeyError(name)

    def __setitem__(self, name, value):
        """Set the state of a child object."""
        self[name].set_state(value)

    def __delitem__(self, name):
        """Delete a child object."""
        for cname in self.child_names:
            cobj = getattr(self, cname)
            try:
                del cobj[name]
                return
            except Exception:
                pass
        raise KeyError(name)

    def __iter__(self):
        """Iterator for child named objects."""
        for cname in self.child_names:
            try:
                for item in getattr(self, cname):
                    yield item
            except Exception:
                continue

    def __len__(self):
        """Number of child named objects."""
        l = 0
        for cname in self.child_names:
            cobj = getattr(self, cname)
            if isinstance(cobj, NamedObject):
                l += len(cobj)
        return l


class _CreatableNamedObjectMixin(collections.abc.MutableMapping, Generic[ChildTypeT]):
    def create(self, name: str) -> ChildTypeT:
        """Create a named object.

        Parameters
        ----------
        name: str
            Name of the new object.

        Returns
        -------
        Object
            Object that has been created.
        """
        self.flproxy.create(self.path, name)
        return self._create_child_object(name)

    def __setitem__(self, name: str, value):
        if name not in self.get_object_names():
            self.flproxy.create(self.path, name)
        child = self._objects.get(name)
        if not child:
            child = self._create_child_object(name)
        child.set_state(value)


class _NonCreatableNamedObjectMixin(
    collections.abc.MutableMapping, Generic[ChildTypeT]
):
    def __setitem__(self, name: str, value):
        if name not in self.get_object_names():
            raise KeyError(name)
        child = self._objects.get(name)
        if not child:
            child = self._create_child_object(name)
        child.set_state(value)


def get_cls(name, info, parent=None, version=None):
    """Create a class for the object identified by "path"."""
    try:
        if name == "":
            pname = "root"
        else:
            pname = to_python_name(name)
        obj_type = info["type"]
        base = _baseTypes.get(obj_type)
        if base is None:
            LOG.error(
                f"Unable to find base class for '{name}' "
                f"(type = '{obj_type}'). "
                f"Falling back to String."
            )
            base = String
        dct = {"fluent_name": name}
        helpinfo = info.get("help")
        if helpinfo:
            dct["__doc__"] = _clean_helpinfo(helpinfo)
        else:
            if parent is None:
                dct["__doc__"] = "'root' object."
            else:
                if obj_type == "command":
                    dct["__doc__"] = f"'{pname.strip('_')}' command."
                elif obj_type == "query":
                    dct["__doc__"] = f"'{pname.strip('_')}' query."
                else:
                    dct["__doc__"] = f"'{pname.strip('_')}' child."

        include_child_named_objects = info.get(
            "include-child-named-objects?", False
        ) or info.get("include_child_named_objects", False)
        user_creatable = info.get("user-creatable?", False) or info.get(
            "user_creatable", False
        )
        if version == "222":
            user_creatable = True

        bases = (base,)
        if include_child_named_objects:
            bases = bases + (_ChildNamedObjectAccessorMixin,)
        if obj_type == "named-object" and user_creatable:
            bases = bases + (_CreatableNamedObjectMixin,)
        elif obj_type == "named-object":
            bases = bases + (_NonCreatableNamedObjectMixin,)

        cls = type(pname, bases, dct)

        taboo = set(dir(cls))
        taboo |= set(
            ["child_names", "command_names", "argument_names", "child_object_type"]
        )

        doc = ""

        def _process_cls_names(info_dict, names, write_doc=False):
            nonlocal taboo
            nonlocal cls

            for cname, cinfo in info_dict.items():
                ccls = get_cls(cname, cinfo, cls, version=version)
                ccls_name = ccls.__name__

                i = 0
                if write_doc:
                    nonlocal doc
                    th = ccls._state_type
                    th = th.__name__ if hasattr(th, "__name__") else str(th)
                    doc += f"    {ccls.__name__} : {th}\n"
                    doc += f"        {ccls.__doc__}\n"

                while ccls_name in taboo:
                    if i > 0:
                        ccls_name = ccls_name[: ccls_name.rfind("_")]
                    i += 1
                    ccls_name += f"_{str(i)}"
                ccls.__name__ = ccls_name
                names.append(ccls.__name__)
                taboo.add(ccls_name)
                setattr(cls, ccls.__name__, ccls)

        children = info.get("children")
        if children:
            taboo.add("child_names")
            cls.child_names = []
            _process_cls_names(children, cls.child_names)

        commands = info.get("commands")
        if commands:
            cls.command_names = []
            _process_cls_names(commands, cls.command_names)

        queries = info.get("queries")
        if queries:
            cls.query_names = []
            _process_cls_names(queries, cls.query_names)

        arguments = info.get("arguments")
        if arguments:
            doc = cls.__doc__
            doc += "\n\n"
            doc += "Parameters\n"
            doc += "----------\n"
            cls.argument_names = []
            _process_cls_names(arguments, cls.argument_names, write_doc=True)
            cls.__doc__ = doc

        object_type = info.get("object-type", False) or info.get("object_type", False)
        if object_type:
            cls.child_object_type = get_cls(
                "child-object-type", object_type, cls, version=version
            )
            cls.child_object_type.rename = lambda self, name: self._parent.rename(
                name, self._name
            )
            cls.child_object_type.get_name = lambda self: self._name
    except Exception:
        print(
            f"Unable to construct class for '{name}' of "
            f"'{parent.fluent_name if parent else None}'"
        )
        raise
    return cls


def _gethash(obj_info):
    dhash = hashlib.sha256()
    dhash.update(pickle.dumps(obj_info))
    return dhash.hexdigest()


def get_root(flproxy, version: str = "") -> Group:
    """Get the root settings object.

    Parameters
    ----------
    flproxy: Proxy
        Object that interfaces with the Fluent backend.

    Returns
    -------
    root object
    """
    obj_info = flproxy.get_static_info()
    try:
        settings = importlib.import_module(
            f"ansys.fluent.core.solver.settings_{version}"
        )

        if settings.SHASH != _gethash(obj_info):
            LOG.warning(
                "Mismatch between generated file and server object "
                "info. Dynamically created settings classes will "
                "be used."
            )
            raise RuntimeError("Mismatch in hash values")
        cls = settings.root
    except Exception:
        cls = get_cls("", obj_info, version=version)
    root = cls()
    root.set_flproxy(flproxy)
    root._setattr("_static_info", obj_info)
    return root



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .table_name import table_name
from .column_with_diameters import column_with_diameters
from .column_with_number_fractions import column_with_number_fractions
from .column_with_mass_fractions import column_with_mass_fractions
from .accumulated_number_fraction import accumulated_number_fraction
from .accumulated_mass_fraction import accumulated_mass_fraction
class tabulated_size_settings(Group):
    """
    'tabulated_size_settings' child.
    """

    fluent_name = "tabulated-size-settings"

    child_names = \
        ['table_name', 'column_with_diameters',
         'column_with_number_fractions', 'column_with_mass_fractions',
         'accumulated_number_fraction', 'accumulated_mass_fraction']

    table_name: table_name = table_name
    """
    table_name child of tabulated_size_settings.
    """
    column_with_diameters: column_with_diameters = column_with_diameters
    """
    column_with_diameters child of tabulated_size_settings.
    """
    column_with_number_fractions: column_with_number_fractions = column_with_number_fractions
    """
    column_with_number_fractions child of tabulated_size_settings.
    """
    column_with_mass_fractions: column_with_mass_fractions = column_with_mass_fractions
    """
    column_with_mass_fractions child of tabulated_size_settings.
    """
    accumulated_number_fraction: accumulated_number_fraction = accumulated_number_fraction
    """
    accumulated_number_fraction child of tabulated_size_settings.
    """
    accumulated_mass_fraction: accumulated_mass_fraction = accumulated_mass_fraction
    """
    accumulated_mass_fraction child of tabulated_size_settings.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .display import display
from .lic_child import lic_child

class lic(NamedObject[lic_child], _CreatableNamedObjectMixin[lic_child]):
    """
    'lic' child.
    """

    fluent_name = "lic"

    command_names = \
        ['display']

    display: display = display
    """
    display command of lic.
    """
    child_object_type: lic_child = lic_child
    """
    child_object_type of lic.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .local_dt_child import local_dt_child

class global_dt(NamedObject[local_dt_child], _CreatableNamedObjectMixin[local_dt_child]):
    """
    'global_dt' child.
    """

    fluent_name = "global-dt"

    child_object_type: local_dt_child = local_dt_child
    """
    child_object_type of global_dt.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

"""A package providing Fluent's Settings Objects in Python."""
from ansys.fluent.core.solver.flobject import *

SHASH = "e4ab2a592442a8504847b7e3b88f1f1e69b1dc30c15bc28f937c93b8eb0160a5"
from .root import root